@@ -1,4 +1,4 @@
-/* $Id: 32bpp_simple.cpp 18873 2010-01-21 01:44:51Z rubidium $ */
+/* $Id: 32bpp_simple.cpp 21782 2011-01-14 15:34:33Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -91,7 +91,7 @@ void Blitter_32bppSimple::DrawColourMappingRect(void *dst, int width, int height
 	DEBUG(misc, 0, "32bpp blitter doesn't know how to draw this colour table ('%d')", pal);
 }
 
-Sprite *Blitter_32bppSimple::Encode(SpriteLoader::Sprite *sprite, Blitter::AllocatorProc *allocator)
+Sprite *Blitter_32bppSimple::Encode(SpriteLoader::Sprite *sprite, AllocatorProc *allocator)
 {
 	Sprite *dest_sprite;
 	SpriteLoader::CommonPixel *dst;
