@@ -1,4 +1,4 @@
-/* $Id: date_func.h 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: date_func.h 20591 2010-08-22 09:18:01Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -18,8 +18,9 @@ extern Year      _cur_year;
 extern Month     _cur_month;
 extern Date      _date;
 extern DateFract _date_fract;
+extern uint16 _tick_counter;
 
-void SetDate(Date date);
+void SetDate(Date date, DateFract fract);
 void ConvertDateToYMD(Date date, YearMonthDay *ymd);
 Date ConvertYMDToDate(Year year, Month month, Day day);
 
