@@ -1,4 +1,4 @@
-/* $Id: economy.cpp 20431 2010-08-09 21:53:44Z rubidium $ */
+/* $Id: economy.cpp 23052 2011-10-22 20:54:23Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -20,17 +20,17 @@
 #include "vehicle_gui.h"
 #include "ai/ai.hpp"
 #include "aircraft.h"
-#include "train.h"
-#include "newgrf_cargo.h"
 #include "newgrf_engine.h"
+#include "engine_base.h"
+#include "ground_vehicle.hpp"
+#include "newgrf_cargo.h"
 #include "newgrf_sound.h"
-#include "newgrf_industries.h"
 #include "newgrf_industrytiles.h"
 #include "newgrf_station.h"
-#include "unmovable.h"
+#include "newgrf_airporttiles.h"
+#include "object.h"
 #include "group.h"
 #include "strings_func.h"
-#include "functions.h"
 #include "window_func.h"
 #include "date_func.h"
 #include "vehicle_func.h"
@@ -45,10 +45,9 @@
 #include "economy_base.h"
 #include "core/pool_func.hpp"
 #include "newgrf.h"
-#include "engine_base.h"
+#include "core/backup_type.hpp"
 
 #include "table/strings.h"
-#include "table/sprites.h"
 #include "table/pricebase.h"
 
 
@@ -72,36 +71,22 @@ static inline int32 BigMulS(const int32 a, const int32 b, const uint8 shift)
 	return (int32)((int64)a * (int64)b >> shift);
 }
 
-/**
- * Multiply two unsigned integers and shift the results to right.
- *
- * This function multiplies two unsigned integers. The result is
- * shifted by the amount of shift to right.
- *
- * @param a The first unsigned integer
- * @param b The second unsigned integer
- * @param shift The amount to shift the value to right.
- * @return The shifted result
- */
-static inline uint32 BigMulSU(const uint32 a, const uint32 b, const uint8 shift)
-{
-	return (uint32)((uint64)a * (uint64)b >> shift);
-}
-
 typedef SmallVector<Industry *, 16> SmallIndustryList;
 
-/* Score info */
+/**
+ * Score info, values used for computing the detailed performance rating.
+ */
 const ScoreInfo _score_info[] = {
-	{ SCORE_VEHICLES,        120, 100 },
-	{ SCORE_STATIONS,         80, 100 },
-	{ SCORE_MIN_PROFIT,    10000, 100 },
-	{ SCORE_MIN_INCOME,    50000,  50 },
-	{ SCORE_MAX_INCOME,   100000, 100 },
-	{ SCORE_DELIVERED,     40000, 400 },
-	{ SCORE_CARGO,             8,  50 },
-	{ SCORE_MONEY,      10000000,  50 },
-	{ SCORE_LOAN,         250000,  50 },
-	{ SCORE_TOTAL,             0,   0 }
+	{     120, 100}, // SCORE_VEHICLES
+	{      80, 100}, // SCORE_STATIONS
+	{   10000, 100}, // SCORE_MIN_PROFIT
+	{   50000,  50}, // SCORE_MIN_INCOME
+	{  100000, 100}, // SCORE_MAX_INCOME
+	{   40000, 400}, // SCORE_DELIVERED
+	{       8,  50}, // SCORE_CARGO
+	{10000000,  50}, // SCORE_MONEY
+	{  250000,  50}, // SCORE_LOAN
+	{       0,   0}  // SCORE_TOTAL
 };
 
 int _score_part[MAX_COMPANIES][SCORE_END];
@@ -122,7 +107,6 @@ static PriceMultipliers _price_base_multiplier;
 Money CalculateCompanyValue(const Company *c, bool including_loan)
 {
 	Owner owner = c->index;
-	Money value = 0;
 
 	Station *st;
 	uint num = 0;
@@ -131,7 +115,7 @@ Money CalculateCompanyValue(const Company *c, bool including_loan)
 		if (st->owner == owner) num += CountBits((byte)st->facilities);
 	}
 
-	value += num * _price[PR_STATION_VALUE] * 25;
+	Money value = num * _price[PR_STATION_VALUE] * 25;
 
 	Vehicle *v;
 	FOR_ALL_VEHICLES(v) {
@@ -152,12 +136,14 @@ Money CalculateCompanyValue(const Company *c, bool including_loan)
 	return max(value, (Money)1);
 }
 
-/** if update is set to true, the economy is updated with this score
+/**
+ * if update is set to true, the economy is updated with this score
  *  (also the house is updated, should only be true in the on-tick event)
  * @param update the economy with calculated score
  * @param c company been evaluated
  * @return actual score of this company
- * */
+ *
+ */
 int UpdateCompanyRatingAndValue(Company *c, bool update)
 {
 	Owner owner = c->index;
@@ -190,8 +176,9 @@ int UpdateCompanyRatingAndValue(Company *c, bool update)
 
 		_score_part[owner][SCORE_VEHICLES] = num;
 		/* Don't allow negative min_profit to show */
-		if (min_profit > 0)
+		if (min_profit > 0) {
 			_score_part[owner][SCORE_MIN_PROFIT] = ClampToI32(min_profit);
+		}
 	}
 
 	/* Count stations */
@@ -286,7 +273,7 @@ int UpdateCompanyRatingAndValue(Company *c, bool update)
 
 	if (update) {
 		c->old_economy[0].performance_history = score;
-		UpdateCompanyHQ(c, score);
+		UpdateCompanyHQ(c->location_of_HQ, score);
 		c->old_economy[0].company_value = CalculateCompanyValue(c);
 	}
 
@@ -297,13 +284,31 @@ int UpdateCompanyRatingAndValue(Company *c, bool update)
 /*  use INVALID_OWNER as new_owner to delete the company. */
 void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 {
+	/* We need to set _current_company to old_owner before we try to move
+	 * the client. This is needed as it needs to know whether "you" really
+	 * are the current local company. */
+	Backup<CompanyByte> cur_company(_current_company, old_owner, FILE_LINE);
 #ifdef ENABLE_NETWORK
 	/* In all cases, make spectators of clients connected to that company */
 	if (_networking) NetworkClientsToSpectators(old_owner);
 #endif /* ENABLE_NETWORK */
+	if (old_owner == _local_company) {
+		/* Single player cheated to AI company.
+		 * There are no specatators in single player, so we must pick some other company. */
+		assert(!_networking);
+		Backup<CompanyByte> cur_company(_current_company, FILE_LINE);
+		Company *c;
+		FOR_ALL_COMPANIES(c) {
+			if (c->index != old_owner) {
+				SetLocalCompany(c->index);
+				break;
+			}
+		}
+		cur_company.Restore();
+		assert(old_owner != _local_company);
+	}
 
 	Town *t;
-	CompanyID old = _current_company;
 
 	assert(old_owner != new_owner);
 
@@ -312,12 +317,11 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 		uint i;
 
 		/* See if the old_owner had shares in other companies */
-		_current_company = old_owner;
 		FOR_ALL_COMPANIES(c) {
 			for (i = 0; i < 4; i++) {
 				if (c->share_owners[i] == old_owner) {
 					/* Sell his shares */
-					CommandCost res = DoCommand(0, c->index, 0, DC_EXEC, CMD_SELL_SHARE_IN_COMPANY);
+					CommandCost res = DoCommand(0, c->index, 0, DC_EXEC | DC_BANKRUPT, CMD_SELL_SHARE_IN_COMPANY);
 					/* Because we are in a DoCommand, we can't just execute another one and
 					 *  expect the money to be removed. We need to do it ourself! */
 					SubtractMoneyFromCompany(res);
@@ -326,21 +330,21 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 		}
 
 		/* Sell all the shares that people have on this company */
+		Backup<CompanyByte> cur_company2(_current_company, FILE_LINE);
 		c = Company::Get(old_owner);
 		for (i = 0; i < 4; i++) {
-			_current_company = c->share_owners[i];
+			cur_company2.Change(c->share_owners[i]);
 			if (_current_company != INVALID_OWNER) {
 				/* Sell the shares */
-				CommandCost res = DoCommand(0, old_owner, 0, DC_EXEC, CMD_SELL_SHARE_IN_COMPANY);
+				CommandCost res = DoCommand(0, old_owner, 0, DC_EXEC | DC_BANKRUPT, CMD_SELL_SHARE_IN_COMPANY);
 				/* Because we are in a DoCommand, we can't just execute another one and
 				 *  expect the money to be removed. We need to do it ourself! */
 				SubtractMoneyFromCompany(res);
 			}
 		}
+		cur_company2.Restore();
 	}
 
-	_current_company = old_owner;
-
 	/* Temporarily increase the company's money, to be sure that
 	 * removing his/her property doesn't fail because of lack of money.
 	 * Not too drastically though, because it could overflow */
@@ -396,6 +400,9 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 					v->colourmap = PAL_NONE;
 					if (v->IsEngineCountable()) Company::Get(new_owner)->num_engines[v->engine_type]++;
 					if (v->IsPrimaryVehicle()) v->unitnumber = unitidgen[v->type].NextID();
+
+					/* Invalidate the vehicle's cargo payment "owner cache". */
+					if (v->cargo_payment != NULL) v->cargo_payment->owner = NULL;
 				}
 			}
 		}
@@ -471,11 +478,15 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 	/* Change colour of existing windows */
 	if (new_owner != INVALID_OWNER) ChangeWindowOwner(old_owner, new_owner);
 
-	_current_company = old;
+	cur_company.Restore();
 
 	MarkWholeScreenDirty();
 }
 
+/**
+ * Check for bankruptcy of a company. Called every three months.
+ * @param c Company to check.
+ */
 static void CompanyCheckBankrupt(Company *c)
 {
 	/*  If the company has money again, it does not go bankrupt */
@@ -487,22 +498,22 @@ static void CompanyCheckBankrupt(Company *c)
 
 	c->quarters_of_bankruptcy++;
 
-	CompanyNewsInformation *cni = MallocT<CompanyNewsInformation>(1);
-	cni->FillData(c);
-
 	switch (c->quarters_of_bankruptcy) {
 		case 0:
 		case 1:
-			free(cni);
 			break;
 
-		case 2:
+		case 2: {
+			CompanyNewsInformation *cni = MallocT<CompanyNewsInformation>(1);
+			cni->FillData(c);
 			SetDParam(0, STR_NEWS_COMPANY_IN_TROUBLE_TITLE);
 			SetDParam(1, STR_NEWS_COMPANY_IN_TROUBLE_DESCRIPTION);
 			SetDParamStr(2, cni->company_name);
 			AddCompanyNewsItem(STR_MESSAGE_NEWS_FORMAT, NS_COMPANY_TROUBLE, cni);
 			AI::BroadcastNewEvent(new AIEventCompanyInTrouble(c->index));
 			break;
+		}
+
 		case 3: {
 			/* Check if the company has any value.. if not, declare it bankrupt
 			 *  right now */
@@ -511,10 +522,9 @@ static void CompanyCheckBankrupt(Company *c)
 				c->bankrupt_value = val;
 				c->bankrupt_asked = 1 << c->index; // Don't ask the owner
 				c->bankrupt_timeout = 0;
-				free(cni);
 				break;
 			}
-			/* Else, falltrue to case 4... */
+			/* FALL THROUGH to case 4... */
 		}
 		default:
 		case 4:
@@ -524,43 +534,43 @@ static void CompanyCheckBankrupt(Company *c)
 				 * he/she is no long in control of this company. However... when you
 				 * join another company (cheat) the "unowned" company can bankrupt. */
 				c->bankrupt_asked = MAX_UVALUE(CompanyMask);
-				free(cni);
 				break;
 			}
 
-			/* Close everything the owner has open */
-			DeleteCompanyWindows(c->index);
-
-			/* Show bankrupt news */
-			SetDParam(0, STR_NEWS_COMPANY_BANKRUPT_TITLE);
-			SetDParam(1, STR_NEWS_COMPANY_BANKRUPT_DESCRIPTION);
-			SetDParamStr(2, cni->company_name);
-			AddCompanyNewsItem(STR_MESSAGE_NEWS_FORMAT, NS_COMPANY_BANKRUPT, cni);
-
-			ChangeOwnershipOfCompanyItems(c->index, INVALID_OWNER);
-
-			if (c->is_ai) AI::Stop(c->index);
-
-			CompanyID c_index = c->index;
-			delete c;
-			AI::BroadcastNewEvent(new AIEventCompanyBankrupt(c_index));
+			/* Actually remove the company, but not when we're a network client.
+			 * In case of network clients we will be getting a command from the
+			 * server. It is done in this way as we are called from the
+			 * StateGameLoop which can't change the current company, and thus
+			 * updating the local company triggers an assert later on. In the
+			 * case of a network game the command will be processed at a time
+			 * that changing the current company is okay. In case of single
+			 * player we are sure (the above check) that we are not the local
+			 * company and thus we won't be moved. */
+			if (!_networking || _network_server) DoCommandP(0, 2 | (c->index << 16), CRR_BANKRUPT, CMD_COMPANY_CTRL);
+			break;
 	}
 }
 
+/**
+ * Update the finances of all companies.
+ * Pay for the stations, update the history graph, update ratings and company values, and deal with bankruptcy.
+ */
 static void CompaniesGenStatistics()
 {
 	Station *st;
-	Company *c;
 
+	Backup<CompanyByte> cur_company(_current_company, FILE_LINE);
 	FOR_ALL_STATIONS(st) {
-		_current_company = st->owner;
+		cur_company.Change(st->owner);
 		CommandCost cost(EXPENSES_PROPERTY, _price[PR_STATION_VALUE] >> 1);
 		SubtractMoneyFromCompany(cost);
 	}
+	cur_company.Restore();
 
-	if (!HasBit(1 << 0 | 1 << 3 | 1 << 6 | 1 << 9, _cur_month))
-		return;
+	/* Only run the economic statics and update company stats every 3rd month (1st of quarter). */
+	if (!HasBit(1 << 0 | 1 << 3 | 1 << 6 | 1 << 9, _cur_month)) return;
 
+	Company *c;
 	FOR_ALL_COMPANIES(c) {
 		memmove(&c->old_economy[1], &c->old_economy[0], sizeof(c->old_economy) - sizeof(c->old_economy[0]));
 		c->old_economy[0] = c->cur_economy;
@@ -678,21 +688,23 @@ void RecomputePrices()
 	SetWindowClassesDirty(WC_BUILD_VEHICLE);
 	SetWindowClassesDirty(WC_REPLACE_VEHICLE);
 	SetWindowClassesDirty(WC_VEHICLE_DETAILS);
-	SetWindowDirty(WC_PAYMENT_RATES, 0);
+	InvalidateWindowData(WC_PAYMENT_RATES, 0);
 }
 
+/** Let all companies pay the monthly interest on their loan. */
 static void CompaniesPayInterest()
 {
 	const Company *c;
 
+	Backup<CompanyByte> cur_company(_current_company, FILE_LINE);
 	FOR_ALL_COMPANIES(c) {
-		_current_company = c->index;
+		cur_company.Change(c->index);
 
 		/* Over a year the paid interest should be "loan * interest percentage",
 		 * but... as that number is likely not dividable by 12 (pay each month),
 		 * one needs to account for that in the monthly fee calculations.
 		 * To easily calculate what one should pay "this" month, you calculate
-		 * what (total) should have been paid up to this month and you substract
+		 * what (total) should have been paid up to this month and you subtract
 		 * whatever has been paid in the previous months. This will mean one month
 		 * it'll be a bit more and the other it'll be a bit less than the average
 		 * monthly fee, but on average it will be exact. */
@@ -704,6 +716,7 @@ static void CompaniesPayInterest()
 
 		SubtractMoneyFromCompany(CommandCost(EXPENSES_OTHER, _price[PR_STATION_VALUE] >> 2));
 	}
+	cur_company.Restore();
 }
 
 static void HandleEconomyFluctuations()
@@ -711,7 +724,7 @@ static void HandleEconomyFluctuations()
 	if (_settings_game.difficulty.economy != 0) {
 		/* When economy is Fluctuating, decrease counter */
 		_economy.fluct--;
-	} else if (_economy.fluct <= 0) {
+	} else if (EconomyIsInRecession()) {
 		/* When it's Steady and we are in recession, end it now */
 		_economy.fluct = -12;
 	} else {
@@ -892,8 +905,6 @@ static uint DeliverGoodsToIndustry(const Station *st, CargoID cargo_type, uint n
 		Industry *ind = st->industries_near[i];
 		if (ind->index == source) continue;
 
-		const IndustrySpec *indspec = GetIndustrySpec(ind->type);
-
 		uint cargo_index;
 		for (cargo_index = 0; cargo_index < lengthof(ind->accepts_cargo); cargo_index++) {
 			if (cargo_type == ind->accepts_cargo[cargo_index]) break;
@@ -902,10 +913,7 @@ static uint DeliverGoodsToIndustry(const Station *st, CargoID cargo_type, uint n
 		if (cargo_index >= lengthof(ind->accepts_cargo)) continue;
 
 		/* Check if industry temporarily refuses acceptance */
-		if (HasBit(indspec->callback_mask, CBM_IND_REFUSE_CARGO)) {
-			uint16 res = GetIndustryCallback(CBID_INDUSTRY_REFUSE_CARGO, 0, GetReverseCargoTranslation(cargo_type, indspec->grf_prop.grffile), ind, ind->type, ind->location.tile);
-			if (res == 0) continue;
-		}
+		if (IndustryTemporarilyRefusesCargo(ind, cargo_type)) continue;
 
 		/* Insert the industry into _cargo_delivery_destinations, if not yet contained */
 		_cargo_delivery_destinations.Include(ind);
@@ -1022,8 +1030,7 @@ CargoPayment::~CargoPayment()
 
 	if (this->visual_profit == 0) return;
 
-	CompanyID old_company = _current_company;
-	_current_company = this->front->owner;
+	Backup<CompanyByte> cur_company(_current_company, this->front->owner, FILE_LINE);
 
 	SubtractMoneyFromCompany(CommandCost(this->front->GetExpenseType(true), -this->route_profit));
 	this->front->profit_this_year += this->visual_profit << 8;
@@ -1038,7 +1045,7 @@ CargoPayment::~CargoPayment()
 		ShowFeederIncomeAnimation(this->front->x_pos, this->front->y_pos, this->front->z_pos, this->visual_profit);
 	}
 
-	_current_company = old_company;
+	cur_company.Restore();
 }
 
 /**
@@ -1069,11 +1076,11 @@ void CargoPayment::PayFinalDelivery(const CargoPacket *cp, uint count)
 Money CargoPayment::PayTransfer(const CargoPacket *cp, uint count)
 {
 	Money profit = GetTransportedGoodsIncome(
-		count,
-		/* pay transfer vehicle for only the part of transfer it has done: ie. cargo_loaded_at_xy to here */
-		DistanceManhattan(cp->LoadedAtXY(), Station::Get(this->current_station)->xy),
-		cp->DaysInTransit(),
-		this->ct);
+			count,
+			/* pay transfer vehicle for only the part of transfer it has done: ie. cargo_loaded_at_xy to here */
+			DistanceManhattan(cp->LoadedAtXY(), Station::Get(this->current_station)->xy),
+			cp->DaysInTransit(),
+			this->ct);
 
 	profit = profit * _settings_game.economy.feeder_payment_share / 100;
 
@@ -1102,6 +1109,10 @@ void PrepareUnload(Vehicle *front_v)
 	}
 
 	assert(front_v->cargo_payment == NULL);
+	/* One CargoPayment per vehicle and the vehicle limit equals the
+	 * limit in number of CargoPayments. Can't go wrong. */
+	assert_compile(CargoPaymentPool::MAX_SIZE == VehiclePool::MAX_SIZE);
+	assert(CargoPayment::CanAllocateItem());
 	front_v->cargo_payment = new CargoPayment(front_v);
 }
 
@@ -1117,10 +1128,8 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 {
 	assert(v->current_order.IsType(OT_LOADING));
 
-	assert(v->load_unload_ticks != 0);
-
 	/* We have not waited enough time till the next round of loading/unloading */
-	if (--v->load_unload_ticks != 0) {
+	if (v->load_unload_ticks != 0) {
 		if (_settings_game.order.improved_load && (v->current_order.GetLoadType() & OLFB_FULL_LOAD)) {
 			/* 'Reserve' this cargo for this vehicle, because we were first. */
 			for (; v != NULL; v = v->Next()) {
@@ -1150,6 +1159,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 	bool completely_emptied = true;
 	bool anything_unloaded = false;
 	bool anything_loaded   = false;
+	uint32 full_load_amount = 0;
 	uint32 cargo_not_full  = 0;
 	uint32 cargo_full      = 0;
 
@@ -1164,7 +1174,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 		byte load_amount = e->info.load_amount;
 
 		/* The default loadamount for mail is 1/4 of the load amount for passengers */
-		if (v->type == VEH_AIRCRAFT && !Aircraft::From(v)->IsNormalAircraft()) load_amount = (load_amount + 3) / 4;
+		if (v->type == VEH_AIRCRAFT && !Aircraft::From(v)->IsNormalAircraft()) load_amount = CeilDiv(load_amount, 4);
 
 		if (_settings_game.order.gradual_loading && HasBit(e->info.callback_mask, CBM_VEHICLE_LOAD_AMOUNT)) {
 			uint16 cb_load_amount = GetVehicleCallback(CBID_VEHICLE_LOAD_AMOUNT, 0, 0, v->engine_type, v);
@@ -1182,7 +1192,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 			payment->SetCargo(v->cargo_type);
 
 			if (HasBit(ge->acceptance_pickup, GoodsEntry::ACCEPTANCE) && !(u->current_order.GetUnloadType() & OUFB_TRANSFER)) {
-				/* The cargo has reached it's final destination, the packets may now be destroyed */
+				/* The cargo has reached its final destination, the packets may now be destroyed */
 				remaining = v->cargo.MoveTo<StationCargoList>(NULL, amount_unloaded, VehicleCargoList::MTA_FINAL_DELIVERY, payment, last_visited);
 
 				dirty_vehicle = true;
@@ -1196,7 +1206,10 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 			 * accept cargo that was loaded at the same station. */
 			if ((u->current_order.GetUnloadType() & (OUFB_UNLOAD | OUFB_TRANSFER)) && (!accepted || v->cargo.Count() == cargo_count)) {
 				remaining = v->cargo.MoveTo(&ge->cargo, amount_unloaded, u->current_order.GetUnloadType() & OUFB_TRANSFER ? VehicleCargoList::MTA_TRANSFER : VehicleCargoList::MTA_UNLOAD, payment);
-				SetBit(ge->acceptance_pickup, GoodsEntry::PICKUP);
+				if (!HasBit(ge->acceptance_pickup, GoodsEntry::PICKUP)) {
+					InvalidateWindowData(WC_STATION_LIST, last_visited);
+					SetBit(ge->acceptance_pickup, GoodsEntry::PICKUP);
+				}
 
 				dirty_vehicle = dirty_station = true;
 			} else if (!accepted) {
@@ -1228,16 +1241,25 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 			continue;
 		}
 
-		/* Do not pick up goods when we have no-load set. */
-		if (u->current_order.GetLoadType() & OLFB_NO_LOAD) continue;
+		/* Do not pick up goods when we have no-load set or loading is stopped. */
+		if (u->current_order.GetLoadType() & OLFB_NO_LOAD || HasBit(u->vehicle_flags, VF_STOP_LOADING)) continue;
 
 		/* update stats */
 		int t;
 		switch (u->type) {
-			case VEH_TRAIN:    t = Train::From(u)->tcache.cached_max_speed; break;
-			case VEH_ROAD:     t = u->max_speed / 2;        break;
-			case VEH_SHIP:     t = u->max_speed;            break;
-			case VEH_AIRCRAFT: t = u->max_speed * 10 / 128; break; // convert to old units
+			case VEH_TRAIN: /* FALL THROUGH */
+			case VEH_SHIP:
+				t = u->vcache.cached_max_speed;
+				break;
+
+			case VEH_ROAD:
+				t = u->vcache.cached_max_speed / 2;
+				break;
+
+			case VEH_AIRCRAFT:
+				t = Aircraft::From(u)->GetSpeedOldUnits(); // Convert to old units.
+				break;
+
 			default: NOT_REACHED();
 		}
 
@@ -1261,13 +1283,24 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 			}
 
 			if (cap > count) cap = count;
-			if (_settings_game.order.gradual_loading) cap = min(cap, load_amount);
+			if (_settings_game.order.gradual_loading) {
+				cap = min(cap, load_amount);
+				cap_left = min(cap_left, load_amount);
+			}
 			if (_settings_game.order.improved_load) {
 				/* Don't load stuff that is already 'reserved' for other vehicles */
 				cap = min((uint)cargo_left[v->cargo_type], cap);
+				count = cargo_left[v->cargo_type];
 				cargo_left[v->cargo_type] -= cap;
 			}
 
+			/* Store whether the maximum possible load amount was loaded or not.*/
+			if (count >= (uint)cap_left) {
+				SetBit(full_load_amount, v->cargo_type);
+			} else {
+				ClrBit(full_load_amount, v->cargo_type);
+			}
+
 			if (v->cargo.Empty()) TriggerVehicle(v, VEHICLE_TRIGGER_NEW_CARGO);
 
 			/* TODO: Regarding this, when we do gradual loading, we
@@ -1285,7 +1318,8 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 			st->time_since_load = 0;
 			st->last_vehicle_type = v->type;
 
-			StationAnimationTrigger(st, st->xy, STAT_ANIM_CARGO_TAKEN, v->cargo_type);
+			TriggerStationAnimation(st, st->xy, SAT_CARGO_TAKEN, v->cargo_type);
+			AirportAnimationTrigger(st, AAT_STATION_CARGO_TAKEN, v->cargo_type);
 
 			unloading_time += cap;
 
@@ -1299,7 +1333,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 		}
 	}
 
-	/* Only set completly_emptied, if we just unloaded all remaining cargo */
+	/* Only set completely_emptied, if we just unloaded all remaining cargo */
 	completely_emptied &= anything_unloaded;
 
 	/* We update these variables here, so gradual loading still fills
@@ -1318,6 +1352,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 
 	if (!anything_unloaded) delete payment;
 
+	ClrBit(u->vehicle_flags, VF_STOP_LOADING);
 	if (anything_loaded || anything_unloaded) {
 		if (_settings_game.order.gradual_loading) {
 			/* The time it takes to load one 'slice' of cargo or passengers depends
@@ -1326,6 +1361,12 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 
 			unloading_time = gradual_loading_wait_time[v->type];
 		}
+		/* We loaded less cargo than possible for all cargo types and it's not full
+		 * load and we're not supposed to wait any longer: stop loading. */
+		if (!anything_unloaded && full_load_amount == 0 && !(v->current_order.GetLoadType() & OLFB_FULL_LOAD) &&
+				v->current_order_time >= (uint)max(v->current_order.wait_time - v->lateness_counter, 0)) {
+			SetBit(v->vehicle_flags, VF_STOP_LOADING);
+		}
 	} else {
 		bool finished_loading = true;
 		if (v->current_order.GetLoadType() & OLFB_FULL_LOAD) {
@@ -1333,7 +1374,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 				/* if the aircraft carries passengers and is NOT full, then
 				 * continue loading, no matter how much mail is in */
 				if ((v->type == VEH_AIRCRAFT && IsCargoInClass(v->cargo_type, CC_PASSENGERS) && v->cargo_cap > v->cargo.Count()) ||
-						(cargo_not_full && (cargo_full & ~cargo_not_full) == 0)) { // There are stull non-full cargos
+						(cargo_not_full && (cargo_full & ~cargo_not_full) == 0)) { // There are still non-full cargoes
 					finished_loading = false;
 				}
 			} else if (cargo_not_full != 0) {
@@ -1347,7 +1388,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 
 	if (v->type == VEH_TRAIN) {
 		/* Each platform tile is worth 2 rail vehicles. */
-		int overhang = Train::From(v)->tcache.cached_total_length - st->GetPlatformLength(v->tile) * TILE_SIZE;
+		int overhang = v->GetGroundVehicleCache()->cached_total_length - st->GetPlatformLength(v->tile) * TILE_SIZE;
 		if (overhang > 0) {
 			unloading_time <<= 1;
 			unloading_time += (overhang * unloading_time) / 8;
@@ -1398,14 +1439,36 @@ void LoadUnloadStation(Station *st)
 	/* No vehicle is here... */
 	if (st->loading_vehicles.empty()) return;
 
+	Vehicle *last_loading = NULL;
+	std::list<Vehicle *>::iterator iter;
+
+	/* Check if anything will be loaded at all. Otherwise we don't need to reserve either. */
+	for (iter = st->loading_vehicles.begin(); iter != st->loading_vehicles.end(); ++iter) {
+		Vehicle *v = *iter;
+
+		if ((v->vehstatus & (VS_STOPPED | VS_CRASHED))) continue;
+
+		assert(v->load_unload_ticks != 0);
+		if (--v->load_unload_ticks == 0) last_loading = v;
+	}
+
+	/* We only need to reserve and load/unload up to the last loading vehicle.
+	 * Anything else will be forgotten anyway after returning from this function.
+	 *
+	 * Especially this means we do _not_ need to reserve cargo for a single
+	 * consist in a station which is not allowed to load yet because its
+	 * load_unload_ticks is still not 0.
+	 */
+	if (last_loading == NULL) return;
+
 	int cargo_left[NUM_CARGO];
 
 	for (uint i = 0; i < NUM_CARGO; i++) cargo_left[i] = st->goods[i].cargo.Count();
 
-	std::list<Vehicle *>::iterator iter;
 	for (iter = st->loading_vehicles.begin(); iter != st->loading_vehicles.end(); ++iter) {
 		Vehicle *v = *iter;
 		if (!(v->vehstatus & (VS_STOPPED | VS_CRASHED))) LoadUnloadVehicle(v, cargo_left);
+		if (v == last_loading) break;
 	}
 
 	/* Call the production machinery of industries */
@@ -1416,6 +1479,9 @@ void LoadUnloadStation(Station *st)
 	_cargo_delivery_destinations.Clear();
 }
 
+/**
+ * Monthly update of the economic data (of the companies as well as economic fluctuations).
+ */
 void CompaniesMonthlyLoop()
 {
 	CompaniesGenStatistics();
@@ -1424,16 +1490,11 @@ void CompaniesMonthlyLoop()
 		RecomputePrices();
 	}
 	CompaniesPayInterest();
-	/* Reset the _current_company flag */
-	_current_company = OWNER_NONE;
 	HandleEconomyFluctuations();
 }
 
 static void DoAcquireCompany(Company *c)
 {
-	Company *owner;
-	int i;
-	Money value;
 	CompanyID ci = c->index;
 
 	CompanyNewsInformation *cni = MallocT<CompanyNewsInformation>(1);
@@ -1450,20 +1511,10 @@ static void DoAcquireCompany(Company *c)
 	ChangeOwnershipOfCompanyItems(ci, _current_company);
 
 	if (c->bankrupt_value == 0) {
-		owner = Company::Get(_current_company);
+		Company *owner = Company::Get(_current_company);
 		owner->current_loan += c->current_loan;
 	}
 
-	value = CalculateCompanyValue(c) >> 2;
-	CompanyID old_company = _current_company;
-	for (i = 0; i != 4; i++) {
-		if (c->share_owners[i] != COMPANY_SPECTATOR) {
-			_current_company = c->share_owners[i];
-			SubtractMoneyFromCompany(CommandCost(EXPENSES_OTHER, -value));
-		}
-	}
-	_current_company = old_company;
-
 	if (c->is_ai) AI::Stop(c->index);
 
 	DeleteCompanyWindows(ci);
@@ -1477,7 +1528,8 @@ static void DoAcquireCompany(Company *c)
 
 extern int GetAmountOwnedBy(const Company *c, Owner owner);
 
-/** Acquire shares in an opposing company.
+/**
+ * Acquire shares in an opposing company.
  * @param tile unused
  * @param flags type of operation
  * @param p1 company to buy the shares from
@@ -1511,12 +1563,11 @@ CommandCost CmdBuyShareInCompany(TileIndex tile, DoCommandFlag flags, uint32 p1,
 	cost.AddCost(CalculateCompanyValue(c) >> 2);
 	if (flags & DC_EXEC) {
 		OwnerByte *b = c->share_owners;
-		int i;
 
 		while (*b != COMPANY_SPECTATOR) b++; // share owners is guaranteed to contain at least one COMPANY_SPECTATOR
 		*b = _current_company;
 
-		for (i = 0; c->share_owners[i] == _current_company;) {
+		for (int i = 0; c->share_owners[i] == _current_company;) {
 			if (++i == 4) {
 				c->bankrupt_value = 0;
 				DoAcquireCompany(c);
@@ -1524,11 +1575,13 @@ CommandCost CmdBuyShareInCompany(TileIndex tile, DoCommandFlag flags, uint32 p1,
 			}
 		}
 		SetWindowDirty(WC_COMPANY, target_company);
+		CompanyAdminUpdate(c);
 	}
 	return cost;
 }
 
-/** Sell shares in an opposing company.
+/**
+ * Sell shares in an opposing company.
  * @param tile unused
  * @param flags type of operation
  * @param p1 company to sell the shares from
@@ -1541,9 +1594,12 @@ CommandCost CmdSellShareInCompany(TileIndex tile, DoCommandFlag flags, uint32 p1
 	CompanyID target_company = (CompanyID)p1;
 	Company *c = Company::GetIfValid(target_company);
 
-	/* Check if selling shares is allowed (protection against modified clients)
-	 * Cannot sell own shares */
-	if (c == NULL || !_settings_game.economy.allow_shares || _current_company == target_company) return CMD_ERROR;
+	/* Cannot sell own shares */
+	if (c == NULL || _current_company == target_company) return CMD_ERROR;
+
+	/* Check if selling shares is allowed (protection against modified clients).
+	 * However, we must sell shares of companies being closed down. */
+	if (!_settings_game.economy.allow_shares && !(flags & DC_BANKRUPT)) return CMD_ERROR;
 
 	/* Those lines are here for network-protection (clients can be slow) */
 	if (GetAmountOwnedBy(c, _current_company) == 0) return CommandCost();
@@ -1557,11 +1613,13 @@ CommandCost CmdSellShareInCompany(TileIndex tile, DoCommandFlag flags, uint32 p1
 		while (*b != _current_company) b++; // share owners is guaranteed to contain company
 		*b = COMPANY_SPECTATOR;
 		SetWindowDirty(WC_COMPANY, target_company);
+		CompanyAdminUpdate(c);
 	}
 	return CommandCost(EXPENSES_OTHER, cost);
 }
 
-/** Buy up another company.
+/**
+ * Buy up another company.
  * When a competing company is gone bankrupt you get the chance to purchase
  * that company.
  * @todo currently this only works for AI companies
