@@ -1,4 +1,4 @@
-/* $Id: engine.cpp 19926 2010-06-04 21:07:08Z rubidium $ */
+/* $Id: engine.cpp 23254 2011-11-18 20:52:13Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -29,6 +29,7 @@
 #include "engine_func.h"
 #include "engine_base.h"
 #include "company_base.h"
+#include "vehicle_func.h"
 
 #include "table/strings.h"
 #include "table/engines.h"
@@ -38,10 +39,17 @@ INSTANTIATE_POOL_METHODS(Engine)
 
 EngineOverrideManager _engine_mngr;
 
-/** Year that engine aging stops. Engines will not reduce in reliability
- * and no more engines will be introduced */
+/**
+ * Year that engine aging stops. Engines will not reduce in reliability
+ * and no more engines will be introduced
+ */
 static Year _year_engine_aging_stops;
 
+/**
+ * The railtypes that have been or never will be introduced, or
+ * an inverse bitmap of rail types that have to be introduced. */
+static uint16 _introduced_railtypes;
+
 /** Number of engines of each vehicle type in original engine data */
 const uint8 _engine_counts[4] = {
 	lengthof(_orig_rail_vehicle_info),
@@ -72,7 +80,7 @@ Engine::Engine() :
 Engine::Engine(VehicleType type, EngineID base)
 {
 	this->type = type;
-	this->internal_id = base;
+	this->grf_prop.local_id = base;
 	this->list_position = base;
 
 	/* Check if this base engine is within the original engine data range */
@@ -81,6 +89,17 @@ Engine::Engine(VehicleType type, EngineID base)
 		this->info.climates = 0x80;
 		/* Set model life to maximum to make wagons available */
 		this->info.base_life = 0xFF;
+		/* Set road vehicle tractive effort to the default value */
+		if (type == VEH_ROAD) this->u.road.tractive_effort = 0x4C;
+		/* Aircraft must have CT_INVALID as default, as there is no property */
+		if (type == VEH_AIRCRAFT) this->info.cargo_type = CT_INVALID;
+		/* Set visual effect to the default value */
+		switch (type) {
+			case VEH_TRAIN: this->u.rail.visual_effect = VE_DEFAULT; break;
+			case VEH_ROAD:  this->u.road.visual_effect = VE_DEFAULT; break;
+			case VEH_SHIP:  this->u.ship.visual_effect = VE_DEFAULT; break;
+			default: break; // The aircraft, disasters and especially visual effects have no NewGRF configured visual effects
+		}
 		return;
 	}
 
@@ -127,6 +146,15 @@ Engine::~Engine()
 	free(this->name);
 }
 
+/**
+ * Checks whether the engine spec is properly initialised.
+ * @return true if enabled
+ */
+bool Engine::IsEnabled() const
+{
+	return this->info.string_id != STR_NEWGRF_INVALID_ENGINE;
+}
+
 /**
  * Determines whether an engine can carry something.
  * A vehicle cannot carry anything if its capacity is zero, or none of the possible cargos is available in the climate.
@@ -184,7 +212,7 @@ uint Engine::GetDisplayDefaultCapacity(uint16 *mail_capacity) const
 			return GetEngineProperty(this->index, PROP_SHIP_CARGO_CAPACITY, this->u.ship.capacity);
 
 		case VEH_AIRCRAFT: {
-			uint capacity = GetEngineProperty(this->index, PROP_AIRCRAFT_PASSENGER_CAPACITY, this->u.air.passenger_capacity);;
+			uint capacity = GetEngineProperty(this->index, PROP_AIRCRAFT_PASSENGER_CAPACITY, this->u.air.passenger_capacity);
 			CargoID cargo = this->GetDefaultCargoType();
 			if (IsCargoInClass(cargo, CC_PASSENGERS)) {
 				if (mail_capacity != NULL) *mail_capacity = GetEngineProperty(this->index, PROP_AIRCRAFT_MAIL_CAPACITY, this->u.air.mail_capacity);
@@ -203,6 +231,10 @@ uint Engine::GetDisplayDefaultCapacity(uint16 *mail_capacity) const
 	}
 }
 
+/**
+ * Return how much the running costs of this engine are.
+ * @return Yearly running cost of the engine.
+ */
 Money Engine::GetRunningCost() const
 {
 	Price base_price;
@@ -233,9 +265,13 @@ Money Engine::GetRunningCost() const
 		default: NOT_REACHED();
 	}
 
-	return GetPrice(base_price, cost_factor, this->grffile, -8);
+	return GetPrice(base_price, cost_factor, this->grf_prop.grffile, -8);
 }
 
+/**
+ * Return how much a new engine costs.
+ * @return Cost of the engine.
+ */
 Money Engine::GetCost() const
 {
 	Price base_price;
@@ -269,11 +305,11 @@ Money Engine::GetCost() const
 		default: NOT_REACHED();
 	}
 
-	return GetPrice(base_price, cost_factor, this->grffile, -8);
+	return GetPrice(base_price, cost_factor, this->grf_prop.grffile, -8);
 }
 
 /**
- * Returns max speed for display purposes
+ * Returns max speed of the engine for display purposes
  * @return max speed in km-ish/h
  */
 uint Engine::GetDisplayMaxSpeed() const
@@ -282,64 +318,83 @@ uint Engine::GetDisplayMaxSpeed() const
 		case VEH_TRAIN:
 			return GetEngineProperty(this->index, PROP_TRAIN_SPEED, this->u.rail.max_speed);
 
-		case VEH_ROAD:
-			return this->u.road.max_speed / 2;
+		case VEH_ROAD: {
+			uint max_speed = GetEngineProperty(this->index, PROP_ROADVEH_SPEED, 0);
+			return (max_speed != 0) ? max_speed * 2 : this->u.road.max_speed / 2;
+		}
 
 		case VEH_SHIP:
 			return GetEngineProperty(this->index, PROP_SHIP_SPEED, this->u.ship.max_speed) / 2;
 
-		case VEH_AIRCRAFT:
+		case VEH_AIRCRAFT: {
+			uint max_speed = GetEngineProperty(this->index, PROP_AIRCRAFT_SPEED, 0);
+			if (max_speed != 0) {
+				return (max_speed * 128) / 10;
+			}
 			return this->u.air.max_speed;
-
-		default: NOT_REACHED();
-	}
-}
-
-uint Engine::GetPower() const
-{
-	/* Currently only trains have 'power' */
-	switch (this->type) {
-		case VEH_TRAIN:
-			return GetEngineProperty(this->index, PROP_TRAIN_POWER, this->u.rail.power);
+		}
 
 		default: NOT_REACHED();
 	}
 }
 
 /**
- * Returns the weight for display purposes.
+ * Returns the power of the engine for display
+ * and sorting purposes.
+ * Only trains and road vehicles have power
+ * @return power in display units hp
+ */
+uint Engine::GetPower() const
+{
+	/* Only trains and road vehicles have 'power'. */
+	switch (this->type) {
+		case VEH_TRAIN:
+			return GetEngineProperty(this->index, PROP_TRAIN_POWER, this->u.rail.power);
+		case VEH_ROAD:
+			return GetEngineProperty(this->index, PROP_ROADVEH_POWER, this->u.road.power) * 10;
+
+		default: NOT_REACHED();
+	}
+}
+
+/**
+ * Returns the weight of the engine for display purposes.
  * For dual-headed train-engines this is the weight of both heads
  * @return weight in display units metric tons
  */
 uint Engine::GetDisplayWeight() const
 {
-	/* Currently only trains have 'weight' */
+	/* Only trains and road vehicles have 'weight'. */
 	switch (this->type) {
 		case VEH_TRAIN:
 			return GetEngineProperty(this->index, PROP_TRAIN_WEIGHT, this->u.rail.weight) << (this->u.rail.railveh_type == RAILVEH_MULTIHEAD ? 1 : 0);
+		case VEH_ROAD:
+			return GetEngineProperty(this->index, PROP_ROADVEH_WEIGHT, this->u.road.weight) / 4;
 
 		default: NOT_REACHED();
 	}
 }
 
 /**
- * Returns the tractive effort for display purposes.
+ * Returns the tractive effort of the engine for display purposes.
  * For dual-headed train-engines this is the tractive effort of both heads
  * @return tractive effort in display units kN
  */
 uint Engine::GetDisplayMaxTractiveEffort() const
 {
-	/* Currently only trains have 'tractive effort' */
+	/* Only trains and road vehicles have 'tractive effort'. */
 	switch (this->type) {
 		case VEH_TRAIN:
 			return (10 * this->GetDisplayWeight() * GetEngineProperty(this->index, PROP_TRAIN_TRACTIVE_EFFORT, this->u.rail.tractive_effort)) / 256;
+		case VEH_ROAD:
+			return (10 * this->GetDisplayWeight() * GetEngineProperty(this->index, PROP_ROADVEH_TRACTIVE_EFFORT, this->u.road.tractive_effort)) / 256;
 
 		default: NOT_REACHED();
 	}
 }
 
 /**
- * Returns the vehicle's life length in days.
+ * Returns the vehicle's (not model's!) life length in days.
  * @return the life length
  */
 Date Engine::GetLifeLengthInDays() const
@@ -386,7 +441,27 @@ EngineID EngineOverrideManager::GetID(VehicleType type, uint16 grf_local_id, uin
 	return INVALID_ENGINE;
 }
 
-/** Sets cached values in Company::num_vehicles and Group::num_vehicles
+/**
+ * Tries to reset the engine mapping to match the current NewGRF configuration.
+ * This is only possible when there are currently no vehicles in the game.
+ * @return false if resetting failed due to present vehicles.
+ */
+bool EngineOverrideManager::ResetToCurrentNewGRFConfig()
+{
+	const Vehicle *v;
+	FOR_ALL_VEHICLES(v) {
+		if (IsCompanyBuildableVehicleType(v)) return false;
+	}
+
+	/* Reset the engines, they will get new EngineIDs */
+	_engine_mngr.ResetToDefaultMapping();
+	ReloadNewGRFData();
+
+	return true;
+}
+
+/**
+ * Sets cached values in Company::num_vehicles and Group::num_vehicles
  */
 void SetCachedEngineCounts()
 {
@@ -424,19 +499,48 @@ void SetCachedEngineCounts()
 	}
 }
 
+/**
+ * Initialise the engine pool with the data from the original vehicles.
+ */
 void SetupEngines()
 {
+	DeleteWindowByClass(WC_ENGINE_PREVIEW);
 	_engine_pool.CleanPool();
 
 	assert(_engine_mngr.Length() >= _engine_mngr.NUM_DEFAULT_ENGINES);
 	const EngineIDMapping *end = _engine_mngr.End();
 	uint index = 0;
 	for (const EngineIDMapping *eid = _engine_mngr.Begin(); eid != end; eid++, index++) {
+		/* Assert is safe; there won't be more than 256 original vehicles
+		 * in any case, and we just cleaned the pool. */
+		assert(Engine::CanAllocateItem());
 		const Engine *e = new Engine(eid->type, eid->internal_id);
 		assert(e->index == index);
 	}
+
+	_introduced_railtypes = 0;
 }
 
+/**
+ * Check whether the railtypes should be introduced.
+ */
+static void CheckRailIntroduction()
+{
+	/* All railtypes have been introduced. */
+	if (_introduced_railtypes == UINT16_MAX || Company::GetPoolSize() == 0) return;
+
+	/* We need to find the railtypes that are known to all companies. */
+	RailTypes rts = (RailTypes)UINT16_MAX;
+
+	/* We are at, or past the introduction date of the rail. */
+	Company *c;
+	FOR_ALL_COMPANIES(c) {
+		c->avail_railtypes = AddDateIntroducedRailTypes(c->avail_railtypes, _date);
+		rts &= c->avail_railtypes;
+	}
+
+	_introduced_railtypes |= rts;
+}
 
 void ShowEnginePreviewWindow(EngineID engine);
 
@@ -484,6 +588,7 @@ static void CalcEngineReliability(Engine *e)
 	SetWindowClassesDirty(WC_REPLACE_VEHICLE);
 }
 
+/** Compute the value for #_year_engine_aging_stops. */
 void SetYearEngineAgingStops()
 {
 	/* Determine last engine aging year, default to 2050 as previously. */
@@ -508,7 +613,6 @@ void SetYearEngineAgingStops()
 void StartupOneEngine(Engine *e, Date aging_date)
 {
 	const EngineInfo *ei = &e->info;
-	uint32 r;
 
 	e->age = 0;
 	e->flags = 0;
@@ -517,7 +621,7 @@ void StartupOneEngine(Engine *e, Date aging_date)
 	/* Don't randomise the start-date in the first two years after gamestart to ensure availability
 	 * of engines in early starting games.
 	 * Note: TTDP uses fixed 1922 */
-	r = Random();
+	uint32 r = Random();
 	e->intro_date = ei->base_intro <= ConvertYMDToDate(_settings_game.game_creation.starting_year + 2, 0, 1) ? ei->base_intro : (Date)GB(r, 0, 9) + ei->base_intro;
 	if (e->intro_date <= _date) {
 		e->age = (aging_date - e->intro_date) >> 5;
@@ -546,6 +650,10 @@ void StartupOneEngine(Engine *e, Date aging_date)
 	}
 }
 
+/**
+ * Start/initialise all our engines. Must be called whenever there are changes
+ * to the NewGRF config.
+ */
 void StartupEngines()
 {
 	Engine *e;
@@ -562,6 +670,22 @@ void StartupEngines()
 		c->avail_railtypes = GetCompanyRailtypes(c->index);
 		c->avail_roadtypes = GetCompanyRoadtypes(c->index);
 	}
+
+	/* Rail types that are invalid or never introduced are marked as
+	 * being introduced upon start. That way we can easily check whether
+	 * there is any date related introduction that is still going to
+	 * happen somewhere in the future. */
+	for (RailType rt = RAILTYPE_BEGIN; rt != RAILTYPE_END; rt++) {
+		const RailtypeInfo *rti = GetRailTypeInfo(rt);
+		if (rti->label != 0 && IsInsideMM(rti->introduction_date, 0, MAX_DAY)) continue;
+
+		SetBit(_introduced_railtypes, rt);
+	}
+
+	CheckRailIntroduction();
+
+	/* Invalidate any open purchase lists */
+	InvalidateWindowClassesData(WC_BUILD_VEHICLE);
 }
 
 static void AcceptEnginePreview(EngineID eid, CompanyID company)
@@ -572,7 +696,7 @@ static void AcceptEnginePreview(EngineID eid, CompanyID company)
 	SetBit(e->company_avail, company);
 	if (e->type == VEH_TRAIN) {
 		assert(e->u.rail.railtype < RAILTYPE_END);
-		SetBit(c->avail_railtypes, e->u.rail.railtype);
+		c->avail_railtypes = AddDateIntroducedRailTypes(c->avail_railtypes | GetRailTypeInfo(e->u.rail.railtype)->introduces_railtypes, _date);
 	} else if (e->type == VEH_ROAD) {
 		SetBit(c->avail_roadtypes, HasBit(e->info.misc_flags, EF_ROAD_TRAM) ? ROADTYPE_TRAM : ROADTYPE_ROAD);
 	}
@@ -581,18 +705,27 @@ static void AcceptEnginePreview(EngineID eid, CompanyID company)
 	if (company == _local_company) {
 		AddRemoveEngineFromAutoreplaceAndBuildWindows(e->type);
 	}
+
+	/* Update the toolbar. */
+	if (e->type == VEH_ROAD) InvalidateWindowData(WC_BUILD_TOOLBAR, TRANSPORT_ROAD);
+	if (e->type == VEH_SHIP) InvalidateWindowData(WC_BUILD_TOOLBAR, TRANSPORT_WATER);
 }
 
+/**
+ * Get the N-th best company.
+ * @param pp Value N, 1 means best, 2 means second best, etc.
+ * @return N-th best company if it exists, #INVALID_COMPANY otherwise.
+ */
 static CompanyID GetBestCompany(uint8 pp)
 {
-	const Company *c;
-	int32 best_hist;
 	CompanyID best_company;
 	CompanyMask mask = 0;
 
 	do {
-		best_hist = -1;
+		int32 best_hist = -1;
 		best_company = INVALID_COMPANY;
+
+		const Company *c;
 		FOR_ALL_COMPANIES(c) {
 			if (c->block_preview == 0 && !HasBit(mask, c->index) &&
 					c->old_economy[0].performance_history > best_hist) {
@@ -609,8 +742,11 @@ static CompanyID GetBestCompany(uint8 pp)
 	return best_company;
 }
 
+/** Daily check to offer an exclusive engine preview to the companies. */
 void EnginesDailyLoop()
 {
+	CheckRailIntroduction();
+
 	if (_cur_year >= _year_engine_aging_stops) return;
 
 	Engine *e;
@@ -640,7 +776,8 @@ void EnginesDailyLoop()
 	}
 }
 
-/** Accept an engine prototype. XXX - it is possible that the top-company
+/**
+ * Accept an engine prototype. XXX - it is possible that the top-company
  * changes while you are waiting to accept the offer? Then it becomes invalid
  * @param tile unused
  * @param flags operation to perfom
@@ -659,6 +796,11 @@ CommandCost CmdWantEnginePreview(TileIndex tile, DoCommandFlag flags, uint32 p1,
 	return CommandCost();
 }
 
+/**
+ * An engine has become available for general use.
+ * Also handle the exclusive engine preview contract.
+ * @param e Engine generally available as of now.
+ */
 static void NewVehicleAvailable(Engine *e)
 {
 	Vehicle *v;
@@ -702,7 +844,7 @@ static void NewVehicleAvailable(Engine *e)
 		/* maybe make another rail type available */
 		RailType railtype = e->u.rail.railtype;
 		assert(railtype < RAILTYPE_END);
-		FOR_ALL_COMPANIES(c) SetBit(c->avail_railtypes, railtype);
+		FOR_ALL_COMPANIES(c) c->avail_railtypes = AddDateIntroducedRailTypes(c->avail_railtypes | GetRailTypeInfo(e->u.rail.railtype)->introduces_railtypes, _date);
 	} else if (e->type == VEH_ROAD) {
 		/* maybe make another road type available */
 		FOR_ALL_COMPANIES(c) SetBit(c->avail_roadtypes, HasBit(e->info.misc_flags, EF_ROAD_TRAM) ? ROADTYPE_TRAM : ROADTYPE_ROAD);
@@ -713,6 +855,10 @@ static void NewVehicleAvailable(Engine *e)
 	SetDParam(0, GetEngineCategoryName(index));
 	SetDParam(1, index);
 	AddNewsItem(STR_NEWS_NEW_VEHICLE_NOW_AVAILABLE_WITH_TYPE, NS_NEW_VEHICLES, NR_ENGINE, index);
+
+	/* Update the toolbar. */
+	if (e->type == VEH_ROAD) InvalidateWindowData(WC_BUILD_TOOLBAR, TRANSPORT_ROAD);
+	if (e->type == VEH_SHIP) InvalidateWindowData(WC_BUILD_TOOLBAR, TRANSPORT_WATER);
 }
 
 void EnginesMonthlyLoop()
@@ -721,11 +867,14 @@ void EnginesMonthlyLoop()
 		Engine *e;
 		FOR_ALL_ENGINES(e) {
 			/* Age the vehicle */
-			if ((e->flags & ENGINE_AVAILABLE) && e->age != 0xFFFF) {
+			if ((e->flags & ENGINE_AVAILABLE) && e->age != MAX_DAY) {
 				e->age++;
 				CalcEngineReliability(e);
 			}
 
+			/* Do not introduce invalid engines */
+			if (!e->IsEnabled()) continue;
+
 			if (!(e->flags & ENGINE_AVAILABLE) && _date >= (e->intro_date + DAYS_IN_YEAR)) {
 				/* Introduce it to all companies */
 				NewVehicleAvailable(e);
@@ -734,8 +883,9 @@ void EnginesMonthlyLoop()
 				e->flags |= ENGINE_EXCLUSIVE_PREVIEW;
 
 				/* Do not introduce new rail wagons */
-				if (!IsWagon(e->index))
+				if (!IsWagon(e->index)) {
 					e->preview_company_rank = 1; // Give to the company with the highest rating.
+				}
 			}
 		}
 	}
@@ -752,7 +902,8 @@ static bool IsUniqueEngineName(const char *name)
 	return true;
 }
 
-/** Rename an engine.
+/**
+ * Rename an engine.
  * @param tile unused
  * @param flags operation to perfom
  * @param p1 engine ID to rename
@@ -768,7 +919,7 @@ CommandCost CmdRenameEngine(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
 	bool reset = StrEmpty(text);
 
 	if (!reset) {
-		if (strlen(text) >= MAX_LENGTH_ENGINE_NAME_BYTES) return CMD_ERROR;
+		if (Utf8StringLength(text) >= MAX_LENGTH_ENGINE_NAME_CHARS) return CMD_ERROR;
 		if (!IsUniqueEngineName(text)) return_cmd_error(STR_ERROR_NAME_MUST_BE_UNIQUE);
 	}
 
@@ -788,7 +939,8 @@ CommandCost CmdRenameEngine(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
 }
 
 
-/** Check if an engine is buildable.
+/**
+ * Check if an engine is buildable.
  * @param engine  index of the engine to check.
  * @param type    the type the engine should be.
  * @param company index of the company.
@@ -808,7 +960,7 @@ bool IsEngineBuildable(EngineID engine, VehicleType type, CompanyID company)
 	/* check if it's available */
 	if (!HasBit(e->company_avail, company)) return false;
 
-	if (e->info.string_id == STR_NEWGRF_INVALID_ENGINE) return false;
+	if (!e->IsEnabled()) return false;
 
 	if (type == VEH_TRAIN) {
 		/* Check if the rail type is available to this company */
