@@ -1,4 +1,4 @@
-/* $Id: engine_gui.cpp 18966 2010-01-30 18:34:48Z frosch $ */
+/* $Id: engine_gui.cpp 21331 2010-11-26 15:22:18Z alberth $ */
 
 /*
  * This file is part of OpenTTD.
@@ -24,7 +24,8 @@
 
 #include "table/strings.h"
 
-/** Return the category of an engine.
+/**
+ * Return the category of an engine.
  * @param engine Engine to examine.
  * @return String describing the category ("road veh", "train". "airplane", or "ship") of the engine.
  */
@@ -71,11 +72,6 @@ struct EnginePreviewWindow : Window {
 		this->InitNested(desc, window_number);
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
 		if (widget != EPW_QUESTION) return;
@@ -97,7 +93,7 @@ struct EnginePreviewWindow : Window {
 		y = DrawStringMultiLine(r.left, r.right, r.top, y, STR_ENGINE_PREVIEW_MESSAGE, TC_FROMSTRING, SA_CENTER) + WD_PAR_VSEP_WIDE;
 
 		SetDParam(0, engine);
-		DrawString(r.left + WD_FRAMERECT_LEFT, r.right - WD_FRAMERECT_RIGHT, y, STR_ENGINE_NAME, TC_BLACK, SA_CENTER);
+		DrawString(r.left + WD_FRAMERECT_LEFT, r.right - WD_FRAMERECT_RIGHT, y, STR_ENGINE_NAME, TC_BLACK, SA_HOR_CENTER);
 		y += FONT_HEIGHT_NORMAL;
 
 		DrawVehicleEngine(r.left + WD_FRAMERECT_LEFT, r.right - WD_FRAMERECT_RIGHT, this->width >> 1, y + VEHICLE_SPACE / 2, engine, GetEnginePalette(engine, _local_company));
@@ -111,7 +107,7 @@ struct EnginePreviewWindow : Window {
 		switch (widget) {
 			case EPW_YES:
 				DoCommandP(0, this->window_number, 0, CMD_WANT_ENGINE_PREVIEW);
-				/* Fallthrough */
+				/* FALL THROUGH */
 			case EPW_NO:
 				delete this;
 				break;
@@ -188,17 +184,36 @@ static StringID GetAircraftEngineInfoString(const Engine *e)
 
 static StringID GetRoadVehEngineInfoString(const Engine *e)
 {
-	SetDParam(0, e->GetCost());
-	SetDParam(1, e->GetDisplayMaxSpeed());
-	uint capacity = GetTotalCapacityOfArticulatedParts(e->index);
-	if (capacity != 0) {
-		SetDParam(2, e->GetDefaultCargoType());
-		SetDParam(3, capacity);
+	if (_settings_game.vehicle.roadveh_acceleration_model == AM_ORIGINAL) {
+		SetDParam(0, e->GetCost());
+		SetDParam(1, e->GetDisplayMaxSpeed());
+		uint capacity = GetTotalCapacityOfArticulatedParts(e->index);
+		if (capacity != 0) {
+			SetDParam(2, e->GetDefaultCargoType());
+			SetDParam(3, capacity);
+		} else {
+			SetDParam(2, CT_INVALID);
+		}
+		SetDParam(4, e->GetRunningCost());
+		return STR_ENGINE_PREVIEW_COST_MAX_SPEED_CAPACITY_RUNCOST;
 	} else {
-		SetDParam(2, CT_INVALID);
+		SetDParam(0, e->GetCost());
+		SetDParam(2, e->GetDisplayMaxSpeed());
+		SetDParam(3, e->GetPower());
+		SetDParam(1, e->GetDisplayWeight());
+		SetDParam(7, e->GetDisplayMaxTractiveEffort());
+
+		SetDParam(4, e->GetRunningCost());
+
+		uint capacity = GetTotalCapacityOfArticulatedParts(e->index);
+		if (capacity != 0) {
+			SetDParam(5, e->GetDefaultCargoType());
+			SetDParam(6, capacity);
+		} else {
+			SetDParam(5, CT_INVALID);
+		}
+		return STR_ENGINE_PREVIEW_COST_WEIGHT_SPEED_POWER_MAX_TE;
 	}
-	SetDParam(4, e->GetRunningCost());
-	return STR_ENGINE_PREVIEW_COST_MAX_SPEED_CAPACITY_RUNCOST;
 }
 
 static StringID GetShipEngineInfoString(const Engine *e)
@@ -273,7 +288,8 @@ void DrawVehicleEngine(int left, int right, int preferred_x, int y, EngineID eng
 	}
 }
 
-/** Sort all items using quick sort and given 'CompareItems' function
+/**
+ * Sort all items using quick sort and given 'CompareItems' function
  * @param el list to be sorted
  * @param compare function for evaluation of the quicksort
  */
@@ -286,7 +302,8 @@ void EngList_Sort(GUIEngineList *el, EngList_SortTypeFunction compare)
 	QSortT(el->Begin(), size, compare);
 }
 
-/** Sort selected range of items (on indices @ <begin, begin+num_items-1>)
+/**
+ * Sort selected range of items (on indices @ <begin, begin+num_items-1>)
  * @param el list to be sorted
  * @param compare function for evaluation of the quicksort
  * @param begin start of sorting
