@@ -1,4 +1,4 @@
-/* $Id: tcp.h 18809 2010-01-15 16:41:15Z rubidium $ */
+/* $Id: tcp.h 22068 2011-02-12 21:09:34Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -19,6 +19,14 @@
 
 #ifdef ENABLE_NETWORK
 
+/** The states of sending the packets. */
+enum SendPacketsState {
+	SPS_CLOSED,      ///< The connection got closed.
+	SPS_NONE_SENT,   ///< The buffer is still full, so no (parts of) packets could be sent.
+	SPS_PARTLY_SENT, ///< The packets are partly sent; there are more packets to be sent in the queue.
+	SPS_ALL_SENT,    ///< All packets in the queue are sent.
+};
+
 /** Base socket handler for all TCP sockets */
 class NetworkTCPSocketHandler : public NetworkSocketHandler {
 private:
@@ -35,11 +43,12 @@ public:
 	bool IsConnected() const { return this->sock != INVALID_SOCKET; }
 
 	virtual NetworkRecvStatus CloseConnection(bool error = true);
-	void Send_Packet(Packet *packet);
-	bool Send_Packets(bool closing_down = false);
-	bool IsPacketQueueEmpty();
+	virtual void SendPacket(Packet *packet);
+	SendPacketsState SendPackets(bool closing_down = false);
 
-	Packet *Recv_Packet();
+	virtual Packet *ReceivePacket();
+
+	bool CanSendReceive();
 
 	NetworkTCPSocketHandler(SOCKET s = INVALID_SOCKET);
 	~NetworkTCPSocketHandler();
@@ -56,13 +65,8 @@ private:
 	bool killed;                ///< Whether we got killed
 	SOCKET sock;                ///< The socket we're connecting with
 
-	/** The actual connection function */
 	void Connect();
 
-	/**
-	 * Entry point for the new threads.
-	 * @param param the TCPConnecter instance to call Connect on.
-	 */
 	static void ThreadEntry(void *param);
 
 protected:
@@ -70,10 +74,6 @@ protected:
 	NetworkAddress address;
 
 public:
-	/**
-	 * Create a new connecter for the given address
-	 * @param address the (un)resolved address to connect to
-	 */
 	TCPConnecter(const NetworkAddress &address);
 	/** Silence the warnings */
 	virtual ~TCPConnecter() {}
@@ -89,15 +89,7 @@ public:
 	 */
 	virtual void OnFailure() {}
 
-	/**
-	 * Check whether we need to call the callback, i.e. whether we
-	 * have connected or aborted and call the appropriate callback
-	 * for that. It's done this way to ease on the locking that
-	 * would otherwise be needed everywhere.
-	 */
 	static void CheckCallbacks();
-
-	/** Kill all connection attempts. */
 	static void KillAll();
 };
 
