@@ -1,4 +1,4 @@
-/* $Id: newgrf.h 18969 2010-01-30 21:49:22Z peter1138 $ */
+/* $Id: newgrf.h 22995 2011-10-04 20:22:46Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,12 +12,31 @@
 #ifndef NEWGRF_H
 #define NEWGRF_H
 
-#include "newgrf_config.h"
 #include "cargotype.h"
-#include "industry_type.h"
-#include "station_type.h"
 #include "rail_type.h"
-#include "house_type.h"
+
+/**
+ * List of different canal 'features'.
+ * Each feature gets an entry in the canal spritegroup table
+ */
+enum CanalFeature {
+	CF_WATERSLOPE,
+	CF_LOCKS,
+	CF_DIKES,
+	CF_ICON,
+	CF_DOCKS,
+	CF_RIVER_SLOPE,
+	CF_RIVER_EDGE,
+	CF_RIVER_GUI,
+	CF_BUOY,
+	CF_END,
+};
+
+/** Canal properties local to the NewGRF */
+struct CanalProperties {
+	uint8 callback_mask;  ///< Bitmask of canal callbacks that have to be called.
+	uint8 flags;          ///< Flags controlling display.
+};
 
 enum GrfLoadingStage {
 	GLS_FILESCAN,
@@ -29,7 +48,7 @@ enum GrfLoadingStage {
 	GLS_END,
 };
 
-DECLARE_POSTFIX_INCREMENT(GrfLoadingStage);
+DECLARE_POSTFIX_INCREMENT(GrfLoadingStage)
 
 enum GrfMiscBit {
 	GMB_DESERT_TREES_FIELDS    = 0, // Unsupported.
@@ -41,14 +60,14 @@ enum GrfMiscBit {
 };
 
 enum GrfSpecFeature {
-	GSF_TRAIN,
-	GSF_ROAD,
-	GSF_SHIP,
+	GSF_TRAINS,
+	GSF_ROADVEHICLES,
+	GSF_SHIPS,
 	GSF_AIRCRAFT,
-	GSF_STATION,
-	GSF_CANAL,
-	GSF_BRIDGE,
-	GSF_TOWNHOUSE,
+	GSF_STATIONS,
+	GSF_CANALS,
+	GSF_BRIDGES,
+	GSF_HOUSES,
 	GSF_GLOBALVAR,
 	GSF_INDUSTRYTILES,
 	GSF_INDUSTRIES,
@@ -58,7 +77,13 @@ enum GrfSpecFeature {
 	GSF_SIGNALS,
 	GSF_OBJECTS,
 	GSF_RAILTYPES,
+	GSF_AIRPORTTILES,
 	GSF_END,
+
+	GSF_FAKE_TOWNS = GSF_END, ///< Fake town GrfSpecFeature for NewGRF debugging (parent scope)
+	GSF_FAKE_END,             ///< End of the fake features
+
+	GSF_INVALID = 0xFF        ///< An invalid spec feature
 };
 
 static const uint32 INVALID_GRFID = 0xFFFFFFFF;
@@ -98,11 +123,15 @@ struct GRFFile {
 	struct SpriteGroup **spritegroups;
 
 	uint sound_offset;
+	uint16 num_sounds;
 
-	StationSpec **stations;
-	HouseSpec **housespec;
-	IndustrySpec **industryspec;
-	IndustryTileSpec **indtspec;
+	struct StationSpec **stations;
+	struct HouseSpec **housespec;
+	struct IndustrySpec **industryspec;
+	struct IndustryTileSpec **indtspec;
+	struct ObjectSpec **objectspec;
+	struct AirportSpec **airportspec;
+	struct AirportTileSpec **airtspec;
 
 	uint32 param[0x80];
 	uint param_end;  ///< one more than the highest set parameter
@@ -117,8 +146,12 @@ struct GRFFile {
 	RailTypeLabel *railtype_list;
 	RailType railtype_map[RAILTYPE_END];
 
+	CanalProperties canal_local_properties[CF_END]; ///< Canal properties as set by this NewGRF
+
+	struct LanguageMap *language_map; ///< Mappings related to the languages.
+
 	int traininfo_vehicle_pitch;  ///< Vertical offset for draing train images in depot GUI and vehicle details
-	int traininfo_vehicle_width;  ///< Width (in pixels) of a 8/8 train vehicle in depot GUI and vehicle details
+	uint traininfo_vehicle_width; ///< Width (in pixels) of a 8/8 train vehicle in depot GUI and vehicle details
 
 	uint32 grf_features;                     ///< Bitset of GrfSpecFeature the grf uses
 	PriceMultipliers price_base_multipliers; ///< Price base multipliers as set by the grf.
@@ -142,6 +175,7 @@ enum ShoreReplacement {
 
 struct GRFLoadedFeatures {
 	bool has_2CC;             ///< Set if any vehicle is loaded which uses 2cc (two company colours).
+	uint64 used_liveries;     ///< Bitmask of #LiveryScheme used by the defined engines.
 	bool has_newhouses;       ///< Set if there are any newhouses loaded.
 	bool has_newindustries;   ///< Set if there are any newindustries loaded.
 	ShoreReplacement shore;   ///< It which way shore sprites were replaced.
@@ -150,9 +184,11 @@ struct GRFLoadedFeatures {
 /* Indicates which are the newgrf features currently loaded ingame */
 extern GRFLoadedFeatures _loaded_newgrf_features;
 
-void LoadNewGRFFile(GRFConfig *config, uint file_index, GrfLoadingStage stage);
+void LoadNewGRFFile(struct GRFConfig *config, uint file_index, GrfLoadingStage stage);
 void LoadNewGRF(uint load_index, uint file_index);
 void ReloadNewGRFData(); // in saveload/afterload.cpp
+void ResetNewGRFData();
+void ResetPersistentNewGRFData();
 
 void CDECL grfmsg(int severity, const char *str, ...) WARN_FORMAT(2, 3);
 
