@@ -1,4 +1,4 @@
-/* $Id: newgrf_engine.cpp 22653 2011-07-10 20:29:19Z rubidium $ */
+/* $Id: newgrf_engine.cpp 23259 2011-11-18 21:19:18Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -587,7 +587,7 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 			return v->grf_cache.company_information;
 
 		case 0x44: // Aircraft information
-			if (v->type != VEH_AIRCRAFT) return UINT_MAX;
+			if (v->type != VEH_AIRCRAFT || !Aircraft::From(v)->IsNormalAircraft()) return UINT_MAX;
 
 			{
 				const Vehicle *w = v->Next();
@@ -792,7 +792,7 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 			switch (variable - 0x80) {
 				case 0x62: return t->track;
 				case 0x66: return t->railtype;
-				case 0x73: return t->gcache.cached_veh_length;
+				case 0x73: return 0x80 + VEHICLE_LENGTH - t->gcache.cached_veh_length;
 				case 0x74: return t->gcache.cached_power;
 				case 0x75: return GB(t->gcache.cached_power,  8, 24);
 				case 0x76: return GB(t->gcache.cached_power, 16, 16);
