@@ -1,4 +1,4 @@
-/* $Id: newgrf_industries.cpp 19938 2010-06-05 21:13:35Z rubidium $ */
+/* $Id: newgrf_industries.cpp 23795 2012-01-13 21:54:59Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,9 +14,9 @@
 #include "industry.h"
 #include "newgrf.h"
 #include "newgrf_industries.h"
-#include "newgrf_commons.h"
 #include "newgrf_text.h"
 #include "newgrf_town.h"
+#include "newgrf_cargo.h"
 #include "window_func.h"
 #include "town.h"
 #include "company_base.h"
@@ -61,7 +61,7 @@ uint32 GetIndustryIDAtOffset(TileIndex tile, const Industry *i, uint32 cur_grfid
 	IndustryGfx gfx = GetCleanIndustryGfx(tile);
 	const IndustryTileSpec *indtsp = GetIndustryTileSpec(gfx);
 
-	if (gfx < NEW_INDUSTRYOFFSET) { // Does it belongs to an old type?
+	if (gfx < NEW_INDUSTRYTILEOFFSET) { // Does it belongs to an old type?
 		/* It is an old tile.  We have to see if it's been overriden */
 		if (indtsp->grf_prop.override == INVALID_INDUSTRYTILE) { // has it been overridden?
 			return 0xFF << 8 | gfx; // no. Tag FF + the gfx id of that tile
@@ -76,7 +76,7 @@ uint32 GetIndustryIDAtOffset(TileIndex tile, const Industry *i, uint32 cur_grfid
 		}
 	}
 	/* Not an 'old type' tile */
-	if (indtsp->grf_prop.spritegroup != NULL) { // tile has a spritegroup ?
+	if (indtsp->grf_prop.spritegroup[0] != NULL) { // tile has a spritegroup ?
 		if (indtsp->grf_prop.grffile->grfid == cur_grfid) { // same industry, same grf ?
 			return indtsp->grf_prop.local_id;
 		} else {
@@ -100,15 +100,17 @@ static uint32 GetClosestIndustry(TileIndex tile, IndustryType type, const Indust
 	return best_dist;
 }
 
-/** Implementation of both var 67 and 68
+/**
+ * Implementation of both var 67 and 68
  * since the mechanism is almost the same, it is easier to regroup them on the same
  * function.
  * @param param_setID parameter given to the callback, which is the set id, or the local id, in our terminology
  * @param layout_filter on what layout do we filter?
+ * @param town_filter Do we filter on the same town as the current industry?
  * @param current Industry for which the inquiry is made
  * @return the formatted answer to the callback : rr(reserved) cc(count) dddd(manhattan distance of closest sister)
  */
-static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout_filter, const Industry *current)
+static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout_filter, bool town_filter, const Industry *current)
 {
 	uint32 GrfID = GetRegister(0x100);  ///< Get the GRFID of the definition to look for in register 100h
 	IndustryType ind_index;
@@ -123,7 +125,7 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout
 
 		case 0xFFFFFFFF: // current grf
 			GrfID = GetIndustrySpec(current->type)->grf_prop.grffile->grfid;
-			/* Fall through */
+			/* FALL THROUGH */
 
 		default: // use the grfid specified in register 100h
 			SetBit(param_setID, 7); // bit 7 means it is not an old type
@@ -134,17 +136,17 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout
 	/* If the industry type is invalid, there is none and the closest is far away. */
 	if (ind_index >= NUM_INDUSTRYTYPES) return 0 | 0xFFFF;
 
-	if (layout_filter == 0) {
+	if (layout_filter == 0 && !town_filter) {
 		/* If the filter is 0, it could be because none was specified as well as being really a 0.
 		 * In either case, just do the regular var67 */
 		closest_dist = GetClosestIndustry(current->location.tile, ind_index, current);
-		count = GetIndustryTypeCount(ind_index);
+		count = min(Industry::GetIndustryTypeCount(ind_index), UINT8_MAX); // clamp to 8 bit
 	} else {
 		/* Count only those who match the same industry type and layout filter
 		 * Unfortunately, we have to do it manually */
 		const Industry *i;
 		FOR_ALL_INDUSTRIES(i) {
-			if (i->type == ind_index && i != current && i->selected_layout == layout_filter) {
+			if (i->type == ind_index && i != current && (i->selected_layout == layout_filter || layout_filter == 0) && (!town_filter || i->town == current->town)) {
 				closest_dist = min(closest_dist, DistanceManhattan(current->location.tile, i->location.tile));
 				count++;
 			}
@@ -154,12 +156,14 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout
 	return count << 16 | GB(closest_dist, 0, 16);
 }
 
-/** This function implements the industries variables that newGRF defines.
+/**
+ * This function implements the industries variables that newGRF defines.
  * @param object the object that we want to query
  * @param variable that is queried
  * @param parameter unused
  * @param available will return false if ever the variable asked for does not exist
- * @return the value stored in the corresponding variable*/
+ * @return the value stored in the corresponding variable
+ */
 uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte parameter, bool *available)
 {
 	const Industry *industry = object->u.industry.ind;
@@ -185,7 +189,7 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 	}
 
 	if (industry == NULL) {
-		DEBUG(grf, 1, "Unhandled property 0x%X (no available industry) in callback 0x%x", variable, object->callback);
+		DEBUG(grf, 1, "Unhandled variable 0x%X (no available industry) in callback 0x%x", variable, object->callback);
 
 		*available = false;
 		return UINT_MAX;
@@ -233,21 +237,21 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		case 0x46: return industry->construction_date; // Date when built - long format - (in days)
 
 		/* Get industry ID at offset param */
-		case 0x60: return GetIndustryIDAtOffset(GetNearbyTile(parameter, industry->location.tile), industry, object->grffile->grfid);
+		case 0x60: return GetIndustryIDAtOffset(GetNearbyTile(parameter, industry->location.tile, false), industry, object->grffile->grfid);
 
 		/* Get random tile bits at offset param */
 		case 0x61:
-			tile = GetNearbyTile(parameter, tile);
-			return (IsTileType(tile, MP_INDUSTRY) && Industry::GetByTile(tile) == industry) ? GetIndustryRandomBits(tile) : 0;
+			tile = GetNearbyTile(parameter, tile, false);
+			return industry->TileBelongsToIndustry(tile) ? GetIndustryRandomBits(tile) : 0;
 
 		/* Land info of nearby tiles */
-		case 0x62: return GetNearbyIndustryTileInformation(parameter, tile, INVALID_INDUSTRY);
+		case 0x62: return GetNearbyIndustryTileInformation(parameter, tile, INVALID_INDUSTRY, false);
 
 		/* Animation stage of nearby tiles */
 		case 0x63:
-			tile = GetNearbyTile(parameter, tile);
-			if (IsTileType(tile, MP_INDUSTRY) && Industry::GetByTile(tile) == industry) {
-				return GetIndustryAnimationState(tile);
+			tile = GetNearbyTile(parameter, tile, false);
+			if (industry->TileBelongsToIndustry(tile)) {
+				return GetAnimationFrame(tile);
 			}
 			return 0xFFFFFFFF;
 
@@ -261,7 +265,16 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		/* Count of industry, distance of closest instance
 		 * 68 is the same as 67, but with a filtering on selected layout */
 		case 0x67:
-		case 0x68: return GetCountAndDistanceOfClosestInstance(parameter, variable == 0x68 ? GB(GetRegister(0x101), 0, 8) : 0, industry);
+		case 0x68: {
+			byte layout_filter = 0;
+			bool town_filter = false;
+			if (variable == 0x68) {
+				uint32 reg = GetRegister(0x101);
+				layout_filter = GB(reg, 0, 8);
+				town_filter = HasBit(reg, 8);
+			}
+			return GetCountAndDistanceOfClosestInstance(parameter, layout_filter, town_filter, industry);
+		}
 
 		/* Get a variable from the persistent storage */
 		case 0x7C: return industry->psa.Get(parameter);
@@ -326,7 +339,7 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		case 0xB4: return Clamp(industry->last_cargo_accepted_at - DAYS_TILL_ORIGINAL_BASE_YEAR, 0, 65535); // Date last cargo accepted since 1920 (in days)
 	}
 
-	DEBUG(grf, 1, "Unhandled industry property 0x%X", variable);
+	DEBUG(grf, 1, "Unhandled industry variable 0x%X", variable);
 
 	*available = false;
 	return UINT_MAX;
@@ -374,10 +387,7 @@ static void NewIndustryResolver(ResolverObject *res, TileIndex tile, Industry *i
 	res->callback        = CBID_NO_CALLBACK;
 	res->callback_param1 = 0;
 	res->callback_param2 = 0;
-	res->last_value      = 0;
-	res->trigger         = 0;
-	res->reseed          = 0;
-	res->count           = 0;
+	res->ResetState();
 
 	const IndustrySpec *indspec = GetIndustrySpec(type);
 	res->grffile         = (indspec != NULL ? indspec->grf_prop.grffile : NULL);
@@ -393,7 +403,7 @@ uint16 GetIndustryCallback(CallbackID callback, uint32 param1, uint32 param2, In
 	object.callback_param1 = param1;
 	object.callback_param2 = param2;
 
-	group = SpriteGroup::Resolve(GetIndustrySpec(type)->grf_prop.spritegroup, &object);
+	group = SpriteGroup::Resolve(GetIndustrySpec(type)->grf_prop.spritegroup[0], &object);
 	if (group == NULL) return CALLBACK_FAILED;
 
 	return group->GetCallbackResult();
@@ -447,7 +457,18 @@ uint32 IndustryLocationGetVariable(const ResolverObject *object, byte variable,
 	return IndustryGetVariable(object, variable, parameter, available);
 }
 
-bool CheckIfCallBackAllowsCreation(TileIndex tile, IndustryType type, uint itspec_index, uint32 seed)
+/**
+ * Check that the industry callback allows creation of the industry.
+ * @param tile %Tile to build the industry.
+ * @param type Type of industry to build.
+ * @param layout Layout number.
+ * @param seed Seed for the random generator.
+ * @param initial_random_bits The random bits the industry is going to have after construction.
+ * @param founder Industry founder
+ * @param creation_type The circumstances the industry is created under.
+ * @return Succeeded or failed command.
+ */
+CommandCost CheckIfCallBackAllowsCreation(TileIndex tile, IndustryType type, uint layout, uint32 seed, uint16 initial_random_bits, Owner founder, IndustryAvailabilityCallType creation_type)
 {
 	const IndustrySpec *indspec = GetIndustrySpec(type);
 
@@ -457,39 +478,47 @@ bool CheckIfCallBackAllowsCreation(TileIndex tile, IndustryType type, uint itspe
 	Industry ind;
 	ind.index = INVALID_INDUSTRY;
 	ind.location.tile = tile;
-	ind.location.w = 0;
+	ind.location.w = 0; // important to mark the industry invalid
 	ind.type = type;
-	ind.selected_layout = itspec_index;
+	ind.selected_layout = layout;
 	ind.town = ClosestTownFromTile(tile, UINT_MAX);
+	ind.random = initial_random_bits;
+	ind.founder = founder;
 
 	NewIndustryResolver(&object, tile, &ind, type);
 	object.GetVariable = IndustryLocationGetVariable;
 	object.callback = CBID_INDUSTRY_LOCATION;
+	object.callback_param2 = creation_type;
 	_industry_creation_random_bits = seed;
 
-	group = SpriteGroup::Resolve(GetIndustrySpec(type)->grf_prop.spritegroup, &object);
+	group = SpriteGroup::Resolve(GetIndustrySpec(type)->grf_prop.spritegroup[0], &object);
 
 	/* Unlike the "normal" cases, not having a valid result means we allow
 	 * the building of the industry, as that's how it's done in TTDP. */
-	if (group == NULL) return true;
+	if (group == NULL) return CommandCost();
 	uint16 result = group->GetCallbackResult();
-	if (result == 0x400 || result == CALLBACK_FAILED) return true;
-
-	/* Copy some parameters from the registers to the error message text ref. stack */
-	SwitchToErrorRefStack();
-	PrepareTextRefStackUsage(4);
-	SwitchToNormalRefStack();
+	if (result == 0x400 || result == CALLBACK_FAILED) return CommandCost();
 
+	CommandCost res;
 	switch (result) {
-		case 0x401: _error_message = STR_ERROR_SITE_UNSUITABLE; break;
-		case 0x402: _error_message = STR_ERROR_CAN_ONLY_BE_BUILT_IN_RAINFOREST; break;
-		case 0x403: _error_message = STR_ERROR_CAN_ONLY_BE_BUILT_IN_DESERT; break;
-		default: _error_message = GetGRFStringID(indspec->grf_prop.grffile->grfid, 0xD000 + result); break;
+		case 0x401: res = CommandCost(STR_ERROR_SITE_UNSUITABLE); break;
+		case 0x402: res = CommandCost(STR_ERROR_CAN_ONLY_BE_BUILT_IN_RAINFOREST); break;
+		case 0x403: res = CommandCost(STR_ERROR_CAN_ONLY_BE_BUILT_IN_DESERT); break;
+		default:    res = CommandCost(GetGRFStringID(indspec->grf_prop.grffile->grfid, 0xD000 + result)); break;
 	}
 
-	return false;
+	/* Copy some parameters from the registers to the error message text ref. stack */
+	res.UseTextRefStack(4);
+
+	return res;
 }
 
+/**
+ * Check with callback #CBM_IND_AVAILABLE whether the industry can be built.
+ * @param type Industry type to check.
+ * @param creation_type Reason to construct a new industry.
+ * @return If the industry has no callback or allows building, \c true is returned. Otherwise, \c false is returned.
+ */
 bool CheckIfCallBackAllowsAvailability(IndustryType type, IndustryAvailabilityCallType creation_type)
 {
 	const IndustrySpec *indspec = GetIndustrySpec(type);
@@ -530,14 +559,14 @@ void IndustryProductionCallback(Industry *ind, int reason)
 			/* display error message */
 			SetDParamStr(0, spec->grf_prop.grffile->filename);
 			SetDParam(1, spec->name);
-			ShowErrorMessage(STR_NEWGRF_BUGGY, STR_NEWGRF_BUGGY_ENDLESS_PRODUCTION_CALLBACK, 0, 0);
+			ShowErrorMessage(STR_NEWGRF_BUGGY, STR_NEWGRF_BUGGY_ENDLESS_PRODUCTION_CALLBACK, WL_WARNING);
 
 			/* abort the function early, this error isn't critical and will allow the game to continue to run */
 			break;
 		}
 
 		SB(object.callback_param2, 8, 16, loop);
-		const SpriteGroup *tgroup = SpriteGroup::Resolve(spec->grf_prop.spritegroup, &object);
+		const SpriteGroup *tgroup = SpriteGroup::Resolve(spec->grf_prop.spritegroup[0], &object);
 		if (tgroup == NULL || tgroup->type != SGT_INDUSTRY_PRODUCTION) break;
 		const IndustryProductionSpriteGroup *group = (const IndustryProductionSpriteGroup *)tgroup;
 
@@ -558,3 +587,35 @@ void IndustryProductionCallback(Industry *ind, int reason)
 
 	SetWindowDirty(WC_INDUSTRY_VIEW, ind->index);
 }
+
+/**
+ * Resolve a industry's spec and such so we can get a variable.
+ * @param ro    The resolver object to fill.
+ * @param index The industry ID to get the data from.
+ */
+void GetIndustryResolver(ResolverObject *ro, uint index)
+{
+	Industry *i = Industry::Get(index);
+	NewIndustryResolver(ro, i->location.tile, i, i->type);
+}
+
+/**
+ * Check whether an industry temporarily refuses to accept a certain cargo.
+ * @param ind The industry to query.
+ * @param cargo_type The cargo to get information about.
+ * @pre cargo_type is in ind->accepts_cargo.
+ * @return Whether the given industry refuses to accept this cargo type.
+ */
+bool IndustryTemporarilyRefusesCargo(Industry *ind, CargoID cargo_type)
+{
+	assert(cargo_type == ind->accepts_cargo[0] || cargo_type == ind->accepts_cargo[1] || cargo_type == ind->accepts_cargo[2]);
+
+	const IndustrySpec *indspec = GetIndustrySpec(ind->type);
+	if (HasBit(indspec->callback_mask, CBM_IND_REFUSE_CARGO)) {
+		uint16 res = GetIndustryCallback(CBID_INDUSTRY_REFUSE_CARGO,
+				0, GetReverseCargoTranslation(cargo_type, indspec->grf_prop.grffile),
+				ind, ind->type, ind->location.tile);
+		return res == 0;
+	}
+	return false;
+}
