@@ -1,4 +1,4 @@
-/* $Id: openttd.cpp 22886 2011-09-03 18:56:34Z frosch $ */
+/* $Id: openttd.cpp 23795 2012-01-13 21:54:59Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -1268,7 +1268,7 @@ void GameLoop()
 
 #ifdef ENABLE_NETWORK
 	/* Check for UDP stuff */
-	if (_network_available) NetworkUDPGameLoop();
+	if (_network_available) NetworkBackgroundLoop();
 
 	if (_networking && !IsGeneratingWorld()) {
 		/* Multiplayer */
