@@ -1,4 +1,4 @@
-/* $Id: town_sl.cpp 20610 2010-08-24 16:15:42Z frosch $ */
+/* $Id: town_sl.cpp 23434 2011-12-04 19:33:20Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -204,6 +204,10 @@ static void Load_TOWN()
 	while ((index = SlIterateArray()) != -1) {
 		Town *t = new (index) Town();
 		SlObject(t, _town_desc);
+
+		if (t->townnamegrfid == 0 && !IsInsideMM(t->townnametype, SPECSTR_TOWNNAME_START, SPECSTR_TOWNNAME_LAST + 1)) {
+			SlErrorCorrupt("Invalid town name generator");
+		}
 	}
 }
 
