@@ -1,4 +1,11 @@
-/* $Id: settings_type.h 16059 2009-04-14 21:07:33Z rubidium $ */
+/* $Id: settings_type.h 23795 2012-01-13 21:54:59Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file settings_type.h Types related to global configuration settings. */
 
@@ -10,12 +17,25 @@
 #include "transport_type.h"
 #include "network/core/config.h"
 #include "company_type.h"
+#include "openttd.h"
+
+/** Available industry map generation densities. */
+enum IndustryDensity {
+	ID_FUND_ONLY, ///< The game does not build industries.
+	ID_MINIMAL,   ///< Start with just the industries that must be present.
+	ID_VERY_LOW,  ///< Very few industries at game start.
+	ID_LOW,       ///< Few industries at game start.
+	ID_NORMAL,    ///< Normal amount of industries at game start.
+	ID_HIGH,      ///< Many industries at game start.
+
+	ID_END,       ///< Number of industry density settings.
+};
 
 /** Settings related to the difficulty of the game */
 struct DifficultySettings {
 	byte   max_no_competitors;               ///< the number of competitors (AIs)
 	byte   number_towns;                     ///< the amount of towns
-	byte   number_industries;                ///< the amount of industries
+	byte   industry_density;                 ///< The industry density. @see IndustryDensity
 	uint32 max_loan;                         ///< the maximum initial loan
 	byte   initial_interest;                 ///< amount of interest (to pay over the loan)
 	byte   vehicle_costs;                    ///< amount of money spent on vehicle running cost
@@ -34,18 +54,19 @@ struct DifficultySettings {
 
 /** Settings related to the GUI and other stuff that is not saved in the savegame. */
 struct GUISettings {
-	bool   vehicle_speed;                    ///< show vehicle speed
 	bool   sg_full_load_any;                 ///< new full load calculation, any cargo must be full read from pre v93 savegames
-	bool   lost_train_warn;                  ///< if a train can't find its destination, show a warning
+	bool   lost_vehicle_warn;                ///< if a vehicle can't find its destination, show a warning
 	uint8  order_review_system;              ///< perform order reviews on vehicles
 	bool   vehicle_income_warn;              ///< if a vehicle isn't generating income, show a warning
-	bool   status_long_date;                 ///< always show long date in status bar
 	bool   show_finances;                    ///< show finances at end of year
 	bool   sg_new_nonstop;                   ///< ttdpatch compatible nonstop handling read from pre v93 savegames
 	bool   new_nonstop;                      ///< ttdpatch compatible nonstop handling
+	uint8  stop_location;                    ///< what is the default stop location of trains?
 	bool   autoscroll;                       ///< scroll when moving mouse to the edge
 	byte   errmsg_duration;                  ///< duration of error message
+	byte   hover_delay;                      ///< time required to activate a hover event, in seconds
 	bool   link_terraform_toolbar;           ///< display terraform toolbar when displaying rail, road, water and airport toolbars
+	uint8  smallmap_land_colour;             ///< colour used for land and heightmap at the smallmap
 	bool   reverse_scroll;                   ///< right-Click-Scrolling scrolls in the opposite direction
 	bool   smooth_scroll;                    ///< smooth scroll viewports
 	bool   measure_tooltip;                  ///< show a permanent tooltip when dragging tools
@@ -55,10 +76,12 @@ struct GUISettings {
 	uint8  loading_indicators;               ///< show loading indicators
 	uint8  default_rail_type;                ///< the default rail type for the rail GUI
 	uint8  toolbar_pos;                      ///< position of toolbars, 0=left, 1=center, 2=right
+	uint8  statusbar_pos;                    ///< position of statusbar, 0=left, 1=center, 2=right
 	uint8  window_snap_radius;               ///< windows snap at each other if closer than this
 	uint8  window_soft_limit;                ///< soft limit of maximum number of non-stickied non-vital windows (0 = no limit)
-	bool   always_build_infrastructure;      ///< always allow building of infrastructure, even when you do not have the vehicles for it
+	bool   disable_unsuitable_building;      ///< disable infrastructure building when no suitable vehicles are available
 	byte   autosave;                         ///< how often should we do autosaves?
+	bool   threaded_saves;                   ///< should we do threaded saves?
 	bool   keep_all_autosave;                ///< name the autosave in a different way
 	bool   autosave_on_exit;                 ///< save an autosave when you quit the game, but do not ask "Do you really want to quit?"
 	uint8  date_format_in_default_names;     ///< should the default savegame/screenshot name use long dates (31th Dec 2008), short dates (31-12-2008) or ISO dates (2008-12-31)
@@ -67,19 +90,16 @@ struct GUISettings {
 	uint8  right_mouse_btn_emulation;        ///< should we emulate right mouse clicking?
 	uint8  scrollwheel_scrolling;            ///< scrolling using the scroll wheel?
 	uint8  scrollwheel_multiplier;           ///< how much 'wheel' per incoming event from the OS?
+	bool   timetable_arrival_departure;      ///< show arrivals and departures in vehicle timetables
 	bool   left_mouse_btn_scrolling;         ///< left mouse button scroll
 	bool   pause_on_newgame;                 ///< whether to start new games paused or not
 	bool   enable_signal_gui;                ///< show the signal GUI when the signal button is pressed
 	Year   coloured_news_year;               ///< when does newspaper become coloured?
 	bool   timetable_in_ticks;               ///< whether to show the timetable in ticks rather than days
 	bool   quick_goto;                       ///< Allow quick access to 'goto button' in vehicle orders window
-	bool   bridge_pillars;                   ///< show bridge pillars for high bridges
 	bool   auto_euro;                        ///< automatically switch to euro in 2002
 	byte   drag_signals_density;             ///< many signals density
 	Year   semaphore_build_before;           ///< build semaphore signals automatically before this year
-	bool   autorenew;                        ///< should autorenew be enabled for new companies?
-	int16  autorenew_months;                 ///< how many months from EOL of vehicles should autorenew trigger for new companies?
-	int32  autorenew_money;                  ///< how much money before autorenewing for new companies?
 	byte   news_message_timeout;             ///< how much longer than the news message "age" should we keep the message in the history
 	bool   show_track_reservation;           ///< highlight reserved tracks.
 	uint8  default_signal_type;              ///< the signal type to build by default.
@@ -89,20 +109,40 @@ struct GUISettings {
 	bool   station_dragdrop;                 ///< whether drag and drop is enabled for stations
 	bool   station_show_coverage;            ///< whether to highlight coverage area
 	bool   persistent_buildingtools;         ///< keep the building tools active after usage
-	uint8  expenses_layout;                  ///< layout of expenses window
+	bool   expenses_layout;                  ///< layout of expenses window
 
 	uint16 console_backlog_timeout;          ///< the minimum amount of time items should be in the console backlog before they will be removed in ~3 seconds granularity.
 	uint16 console_backlog_length;           ///< the minimum amount of items in the console backlog before items will be removed.
 #ifdef ENABLE_NETWORK
 	uint16 network_chat_box_width;           ///< width of the chat box in pixels
 	uint8  network_chat_box_height;          ///< height of the chat box in lines
+	uint16 network_chat_timeout;             ///< timeout of chat messages in seconds
 #endif
+
+	uint8  developer;                        ///< print non-fatal warnings in console (>= 1), copy debug output to console (== 2)
+	bool   show_date_in_logs;                ///< whether to show dates in console logs
+	bool   newgrf_developer_tools;           ///< activate NewGRF developer tools and allow modifying NewGRFs in an existing game
+	bool   ai_developer_tools;               ///< activate AI developer tools
+	bool   scenario_developer;               ///< activate scenario developer: allow modifying NewGRFs in an existing game
+	bool   newgrf_show_old_versions;         ///< whether to show old versions in the NewGRF list
+
+	/**
+	 * Returns true when the user has sufficient privileges to edit newgrfs on a running game
+	 * @return whether the user has sufficient privileges to edit newgrfs in an existing game
+	 */
+	bool UserIsAllowedToChangeNewGRFs() const
+	{
+		return this->scenario_developer || this->newgrf_developer_tools;
+	}
 };
 
 /** Settings related to currency/unit systems. */
 struct LocaleSettings {
 	byte   currency;                         ///< currency we currently use
 	byte   units;                            ///< unit system we show everything
+	char  *digit_group_separator;            ///< thousand separator for non-currencies
+	char  *digit_group_separator_currency;   ///< thousand separator for currencies
+	char  *digit_decimal_separator;          ///< decimal separator
 };
 
 /** All settings related to the network. */
@@ -110,19 +150,27 @@ struct NetworkSettings {
 #ifdef ENABLE_NETWORK
 	uint16 sync_freq;                                     ///< how often do we check whether we are still in-sync
 	uint8  frame_freq;                                    ///< how often do we send commands to the clients
+	uint16 commands_per_frame;                            ///< how many commands may be sent each frame_freq frames?
+	uint16 max_commands_in_queue;                         ///< how many commands may there be in the incoming queue before dropping the connection?
+	uint16 bytes_per_frame;                               ///< how many bytes may, over a long period, be received per frame?
+	uint16 bytes_per_frame_burst;                         ///< how many bytes may, over a short period, be received?
 	uint16 max_join_time;                                 ///< maximum amount of time, in game ticks, a client may take to join
+	uint16 max_download_time;                             ///< maximum amount of time, in game ticks, a client may take to download the map
+	uint16 max_password_time;                             ///< maximum amount of time, in game ticks, a client may take to enter the password
 	bool   pause_on_join;                                 ///< pause the game when people join
-	char   server_bind_ip[NETWORK_HOSTNAME_LENGTH];       ///< IP address the server binds to
 	uint16 server_port;                                   ///< port the server listens on
+	uint16 server_admin_port;                             ///< port the server listens on for the admin network
+	bool   server_admin_chat;                             ///< allow private chat for the server to be distributed to the admin network
 	char   server_name[NETWORK_NAME_LENGTH];              ///< name of the server
-	char   server_password[NETWORK_PASSWORD_LENGTH];      ///< passowrd for joining this server
-	char   rcon_password[NETWORK_PASSWORD_LENGTH];        ///< passowrd for rconsole (server side)
+	char   server_password[NETWORK_PASSWORD_LENGTH];      ///< password for joining this server
+	char   rcon_password[NETWORK_PASSWORD_LENGTH];        ///< password for rconsole (server side)
+	char   admin_password[NETWORK_PASSWORD_LENGTH];       ///< password for the admin network
 	bool   server_advertise;                              ///< advertise the server to the masterserver
 	uint8  lan_internet;                                  ///< search on the LAN or internet for servers
 	char   client_name[NETWORK_CLIENT_NAME_LENGTH];       ///< name of the player (as client)
 	char   default_company_pass[NETWORK_PASSWORD_LENGTH]; ///< default password for new companies in encrypted form
 	char   connect_to_ip[NETWORK_HOSTNAME_LENGTH];        ///< default for the "Add server" query
-	char   network_id[NETWORK_UNIQUE_ID_LENGTH];          ///< semi-unique ID of the client
+	char   network_id[NETWORK_SERVER_ID_LENGTH];          ///< network ID for servers
 	bool   autoclean_companies;                           ///< automatically remove companies that are not in use
 	uint8  autoclean_unprotected;                         ///< remove passwordless companies after this many months
 	uint8  autoclean_protected;                           ///< remove the password from passworded companies after this many months
@@ -136,6 +184,7 @@ struct NetworkSettings {
 	bool   reload_cfg;                                    ///< reload the config file before restarting
 	char   last_host[NETWORK_HOSTNAME_LENGTH];            ///< IP address of the last joined server
 	uint16 last_port;                                     ///< port of the last joined server
+	bool   no_http_content_downloads;                     ///< do not do content downloads over HTTP
 #else /* ENABLE_NETWORK */
 #endif
 };
@@ -158,19 +207,30 @@ struct GameCreationSettings {
 	byte   snow_line;                        ///< the snowline level in this game
 	byte   water_borders;                    ///< bitset of the borders that are water
 	uint16 custom_town_number;               ///< manually entered number of towns
+	byte   variety;                          ///< variety level applied to TGP
+	byte   custom_sea_level;                 ///< manually entered percentage of water in the map
 };
 
 /** Settings related to construction in-game */
 struct ConstructionSettings {
 	bool   build_on_slopes;                  ///< allow building on slopes
 	bool   autoslope;                        ///< allow terraforming under things
-	bool   longbridges;                      ///< allow 100 tile long bridges
+	uint16 max_bridge_length;                ///< maximum length of bridges
+	uint16 max_tunnel_length;                ///< maximum length of tunnels
 	bool   signal_side;                      ///< show signals on right side
 	bool   extra_dynamite;                   ///< extra dynamite
 	bool   road_stop_on_town_road;           ///< allow building of drive-through road stops on town owned roads
 	bool   road_stop_on_competitor_road;     ///< allow building of drive-through road stops on roads owned by competitors
 	uint8  raw_industry_construction;        ///< type of (raw) industry construction (none, "normal", prospecting)
+	uint8  industry_platform;                ///< the amount of flat land around an industry
 	bool   freeform_edges;                   ///< allow terraforming the tiles at the map edges
+	uint8  extra_tree_placement;             ///< (dis)allow building extra trees in-game
+	uint8  command_pause_level;              ///< level/amount of commands that can't be executed while paused
+
+	uint32 terraform_per_64k_frames;         ///< how many tile heights may, over a long period, be terraformed per 65536 frames?
+	uint16 terraform_frame_burst;            ///< how many tile heights may, over a short period, be terraformed?
+	uint32 clear_per_64k_frames;             ///< how many tiles may, over a long period, be cleared per 65536 frames?
+	uint16 clear_frame_burst;                ///< how many tiles may, over a short period, be cleared?
 };
 
 /** Settings related to the AI. */
@@ -197,6 +257,7 @@ struct NPFSettings {
 	 * of not being perfect anymore.
 	 */
 	uint32 npf_max_search_nodes;
+	uint32 maximum_go_to_depot_penalty;      ///< What is the maximum penalty that may be endured for going to a depot
 
 	uint32 npf_rail_firstred_penalty;        ///< the penalty for when the first signal is red (and it is not an exit or combo signal)
 	uint32 npf_rail_firstred_exit_penalty;   ///< the penalty for when the first signal is red (and it is an exit or combo signal)
@@ -212,12 +273,15 @@ struct NPFSettings {
 	uint32 npf_road_curve_penalty;           ///< the penalty for curves
 	uint32 npf_crossing_penalty;             ///< the penalty for level crossings
 	uint32 npf_road_drive_through_penalty;   ///< the penalty for going through a drive-through road stop
+	uint32 npf_road_dt_occupied_penalty;     ///< the penalty multiplied by the fill percentage of a drive-through road stop
+	uint32 npf_road_bay_occupied_penalty;    ///< the penalty multiplied by the fill percentage of a road bay
 };
 
 /** Settings related to the yet another pathfinder. */
 struct YAPFSettings {
 	bool   disable_node_optimization;        ///< whether to use exit-dir instead of trackdir in node key
 	uint32 max_search_nodes;                 ///< stop path-finding when this number of nodes visited
+	uint32 maximum_go_to_depot_penalty;      ///< What is the maximum penalty that may be endured for going to a depot
 	bool   ship_use_yapf;                    ///< use YAPF for ships
 	bool   road_use_yapf;                    ///< use YAPF for road
 	bool   rail_use_yapf;                    ///< use YAPF for rail
@@ -225,6 +289,8 @@ struct YAPFSettings {
 	uint32 road_curve_penalty;               ///< penalty for curves
 	uint32 road_crossing_penalty;            ///< penalty for level crossing
 	uint32 road_stop_penalty;                ///< penalty for going through a drive-through road stop
+	uint32 road_stop_occupied_penalty;       ///< penalty multiplied by the fill percentage of a drive-through road stop
+	uint32 road_stop_bay_occupied_penalty;   ///< penalty multiplied by the fill percentage of a road bay
 	bool   rail_firstred_twoway_eol;         ///< treat first red two-way signal as dead end
 	uint32 rail_firstred_penalty;            ///< penalty for first red signal
 	uint32 rail_firstred_exit_penalty;       ///< penalty for first red exit signal
@@ -261,6 +327,7 @@ struct PathfinderSettings {
 	bool   roadveh_queue;                    ///< buggy road vehicle queueing
 	bool   forbid_90_deg;                    ///< forbid trains to make 90 deg turns
 
+	bool   reverse_at_signals;               ///< whether to reverse at signals at all
 	byte   wait_oneway_signal;               ///< waitingtime in days before a oneway signal
 	byte   wait_twoway_signal;               ///< waitingtime in days before a twoway signal
 
@@ -278,33 +345,31 @@ struct OrderSettings {
 	bool   improved_load;                    ///< improved loading algorithm
 	bool   gradual_loading;                  ///< load vehicles gradually
 	bool   selectgoods;                      ///< only send the goods to station if a train has been there
-	bool   gotodepot;                        ///< allow goto depot in orders
 	bool   no_servicing_if_no_breakdowns;    ///< dont send vehicles to depot when breakdowns are disabled
-	bool   timetabling;                      ///< whether to allow timetabling
 	bool   serviceathelipad;                 ///< service helicopters at helipads automatically (no need to send to depot)
 };
 
 /** Settings related to vehicles. */
 struct VehicleSettings {
-	bool   mammoth_trains;                   ///< allow very long trains
+	uint8  max_train_length;                 ///< maximum length for trains
+	uint8  smoke_amount;                     ///< amount of smoke/sparks locomotives produce
 	uint8  train_acceleration_model;         ///< realistic acceleration for trains
+	uint8  roadveh_acceleration_model;       ///< realistic acceleration for road vehicles
+	uint8  train_slope_steepness;            ///< Steepness of hills for trains when using realistic acceleration
+	uint8  roadveh_slope_steepness;          ///< Steepness of hills for road vehicles when using realistic acceleration
 	bool   wagon_speed_limits;               ///< enable wagon speed limits
 	bool   disable_elrails;                  ///< when true, the elrails are disabled
 	UnitID max_trains;                       ///< max trains in game per company
 	UnitID max_roadveh;                      ///< max trucks in game per company
 	UnitID max_aircraft;                     ///< max planes in game per company
 	UnitID max_ships;                        ///< max ships in game per company
-	bool   servint_ispercent;                ///< service intervals are in percents
-	uint16 servint_trains;                   ///< service interval for trains
-	uint16 servint_roadveh;                  ///< service interval for road vehicles
-	uint16 servint_aircraft;                 ///< service interval for aircraft
-	uint16 servint_ships;                    ///< service interval for ships
 	uint8  plane_speed;                      ///< divisor for speed of aircraft
 	uint8  freight_trains;                   ///< value to multiply the weight of cargo by
 	bool   dynamic_engines;                  ///< enable dynamic allocation of engine data
 	bool   never_expire_vehicles;            ///< never expire vehicles
 	byte   extend_vehicle_life;              ///< extend vehicle life by this many years
 	byte   road_side;                        ///< the side of the road vehicles drive on
+	uint8  plane_crashes;                    ///< number of plane crashes, 0 = none, 1 = reduced, 2 = normal
 };
 
 /** Settings related to the economy. */
@@ -313,32 +378,51 @@ struct EconomySettings {
 	bool   bribe;                            ///< enable bribing the local authority
 	bool   smooth_economy;                   ///< smooth economy
 	bool   allow_shares;                     ///< allow the buying/selling of shares
+	uint8  feeder_payment_share;             ///< percentage of leg payment to virtually pay in feeder systems
 	byte   dist_local_authority;             ///< distance for town local authority, default 20
 	bool   exclusive_rights;                 ///< allow buying exclusive rights
+	bool   fund_roads;                       ///< allow funding local road reconstruction
 	bool   give_money;                       ///< allow giving other companies money
 	bool   mod_road_rebuild;                 ///< roadworks remove unneccesary RoadBits
 	bool   multiple_industry_per_town;       ///< allow many industries of the same type per town
-	bool   same_industry_close;              ///< allow same type industries to be built close to each other
 	uint8  town_growth_rate;                 ///< town growth rate
 	uint8  larger_towns;                     ///< the number of cities to build. These start off larger and grow twice as fast
 	uint8  initial_city_size;                ///< multiplier for the initial size of the cities compared to towns
-	TownLayoutByte town_layout;              ///< select town layout
+	TownLayoutByte town_layout;              ///< select town layout, @see TownLayout
 	bool   allow_town_roads;                 ///< towns are allowed to build roads (always allowed when generating world / in SE)
+	TownFoundingByte found_town;             ///< town founding, @see TownFounding
 	bool   station_noise_level;              ///< build new airports when the town noise level is still within accepted limits
 	uint16 town_noise_population[3];         ///< population to base decision on noise evaluation (@see town_council_tolerance)
+	bool   allow_town_level_crossings;       ///< towns are allowed to build level crossings
 };
 
 /** Settings related to stations. */
 struct StationSettings {
 	bool   modified_catchment;               ///< different-size catchment areas
-	bool   join_stations;                    ///< allow joining of train stations
-	bool   nonuniform_stations;              ///< allow nonuniform train stations
 	bool   adjacent_stations;                ///< allow stations to be built directly adjacent to other stations
 	bool   distant_join_stations;            ///< allow to join non-adjacent stations
-	bool   always_small_airport;             ///< always allow small airports
+	bool   never_expire_airports;            ///< never expire airports
 	byte   station_spread;                   ///< amount a station may spread
 };
 
+/** Default settings for vehicles. */
+struct VehicleDefaultSettings {
+	bool   servint_ispercent;                ///< service intervals are in percents
+	uint16 servint_trains;                   ///< service interval for trains
+	uint16 servint_roadveh;                  ///< service interval for road vehicles
+	uint16 servint_aircraft;                 ///< service interval for aircraft
+	uint16 servint_ships;                    ///< service interval for ships
+};
+
+/** Settings that can be set per company. */
+struct CompanySettings {
+	bool engine_renew;                       ///< is autorenew enabled
+	int16 engine_renew_months;               ///< months before/after the maximum vehicle age a vehicle should be renewed
+	uint32 engine_renew_money;               ///< minimum amount of money before autorenew is used
+	bool renew_keep_length;                  ///< sell some wagons if after autoreplace the train is longer than before
+	VehicleDefaultSettings vehicle;          ///< default settings for vehicles
+};
+
 /** All settings together for the game. */
 struct GameSettings {
 	DifficultySettings   difficulty;         ///< settings related to the difficulty
@@ -358,6 +442,7 @@ struct GameSettings {
 struct ClientSettings {
 	GUISettings          gui;                ///< settings related to the GUI
 	NetworkSettings      network;            ///< settings related to the network
+	CompanySettings      company;            ///< default values for per-company settings
 };
 
 /** The current settings for this game. */
@@ -369,4 +454,16 @@ extern GameSettings _settings_game;
 /** The settings values that are used for new games and/or modified in config file. */
 extern GameSettings _settings_newgame;
 
+/** Old vehicle settings, which were game settings before, and are company settings now. (Needed for savegame conversion) */
+extern VehicleDefaultSettings _old_vds;
+
+/**
+ * Get the settings-object applicable for the current situation: the newgame settings
+ * when we're in the main menu and otherwise the settings of the current game.
+ */
+static inline GameSettings &GetGameSettings()
+{
+	return (_game_mode == GM_MENU) ? _settings_newgame : _settings_game;
+}
+
 #endif /* SETTINGS_TYPE_H */
