@@ -1,4 +1,4 @@
-/* $Id: sprites.h 19005 2010-02-04 15:42:38Z rubidium $ */
+/* $Id: sprites.h 21806 2011-01-15 16:27:04Z terkhen $ */
 
 /*
  * This file is part of OpenTTD.
@@ -7,7 +7,8 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file sprites.h
+/**
+ * @file sprites.h
  * This file contails all sprite-related enums and defines. These consist mainly of
  * the sprite numbers and a bunch of masks and macros to handle sprites and to get
  * rid of all the magic numbers in the code.
@@ -55,7 +56,7 @@ static const SpriteID SPR_LARGE_SMALL_WINDOW = 682;
 
 /* Extra graphic spritenumbers */
 static const SpriteID SPR_OPENTTD_BASE   = 4896;
-static const uint16 OPENTTD_SPRITE_COUNT = 153;
+static const uint16 OPENTTD_SPRITE_COUNT = 160;
 
 /* Halftile-selection sprites */
 static const SpriteID SPR_HALFTILE_SELECTION_FLAT = SPR_OPENTTD_BASE;
@@ -63,7 +64,7 @@ static const SpriteID SPR_HALFTILE_SELECTION_DOWN = SPR_OPENTTD_BASE + 4;
 static const SpriteID SPR_HALFTILE_SELECTION_UP   = SPR_OPENTTD_BASE + 8;
 
 static const SpriteID SPR_SQUARE             = SPR_OPENTTD_BASE + 38;  // coloured square (used for newgrf compatibility)
-static const SpriteID SPR_BLOT               = SPR_OPENTTD_BASE + 39;  // coloured circle (mainly used as vehicle profit marker and for server compatibility)
+static const SpriteID SPR_BLOT               = SPR_OPENTTD_BASE + 39;  // coloured circle (used for server compatibility and installed content)
 static const SpriteID SPR_LOCK               = SPR_OPENTTD_BASE + 40;  // lock icon (for password protected servers)
 static const SpriteID SPR_BOX_EMPTY          = SPR_OPENTTD_BASE + 41;
 static const SpriteID SPR_BOX_CHECKED        = SPR_OPENTTD_BASE + 42;
@@ -72,6 +73,7 @@ static const SpriteID SPR_WINDOW_RESIZE_RIGHT= SPR_OPENTTD_BASE + 44;  // resize
 static const SpriteID SPR_WINDOW_RESIZE_LEFT = SPR_OPENTTD_BASE + 149; // resize icon to the left
 static const SpriteID SPR_WINDOW_SHADE       = SPR_OPENTTD_BASE + 151; // shade the window icon
 static const SpriteID SPR_WINDOW_UNSHADE     = SPR_OPENTTD_BASE + 152; // unshade the window icon
+static const SpriteID SPR_WINDOW_DEBUG       = SPR_OPENTTD_BASE + 153; // NewGRF debug window icon
 static const SpriteID SPR_IMG_PLAY_MUSIC_RTL = SPR_OPENTTD_BASE + 150; // play music button, but then for RTL users
 /* Arrow icons pointing in all 4 directions */
 static const SpriteID SPR_ARROW_DOWN         = SPR_OPENTTD_BASE + 45;
@@ -114,6 +116,14 @@ static const SpriteID SPR_REPLACE_SHIP      = SPR_OPENTTD_BASE + 103;
 static const SpriteID SPR_REPLACE_AIRCRAFT  = SPR_OPENTTD_BASE + 104;
 static const SpriteID SPR_SELL_CHAIN_TRAIN  = SPR_OPENTTD_BASE + 105;
 
+static const SpriteID SPR_PROFIT_NA         = SPR_OPENTTD_BASE + 154;
+static const SpriteID SPR_PROFIT_NEGATIVE   = SPR_OPENTTD_BASE + 155;
+static const SpriteID SPR_PROFIT_SOME       = SPR_OPENTTD_BASE + 156;
+static const SpriteID SPR_PROFIT_LOT        = SPR_OPENTTD_BASE + 157;
+
+static const SpriteID SPR_UNREAD_NEWS                = SPR_OPENTTD_BASE + 158;
+static const SpriteID SPR_EXCLUSIVE_TRANSPORT        = SPR_OPENTTD_BASE + 159;
+
 static const SpriteID SPR_GROUP_CREATE_TRAIN         = SPR_OPENTTD_BASE + 114;
 static const SpriteID SPR_GROUP_CREATE_ROADVEH       = SPR_OPENTTD_BASE + 115;
 static const SpriteID SPR_GROUP_CREATE_SHIP          = SPR_OPENTTD_BASE + 116;
@@ -207,7 +217,20 @@ static const SpriteID SPR_GRASS_RIGHT       = SPR_AIRPORTX_BASE + 13;
 static const SpriteID SPR_GRASS_LEFT        = SPR_AIRPORTX_BASE + 14;
 static const uint16 AIRPORTX_SPRITE_COUNT = 15;
 
-static const SpriteID SPR_ROADSTOP_BASE     = SPR_AIRPORTX_BASE + AIRPORTX_SPRITE_COUNT; // The sprites used for drive-through road stops
+/* Airport preview sprites */
+static const SpriteID SPR_AIRPORT_PREVIEW_BASE             = SPR_AIRPORTX_BASE + AIRPORTX_SPRITE_COUNT;
+static const SpriteID SPR_AIRPORT_PREVIEW_SMALL            = SPR_AIRPORT_PREVIEW_BASE;
+static const SpriteID SPR_AIRPORT_PREVIEW_LARGE            = SPR_AIRPORT_PREVIEW_BASE + 1;
+static const SpriteID SPR_AIRPORT_PREVIEW_HELIPORT         = SPR_AIRPORT_PREVIEW_BASE + 2;
+static const SpriteID SPR_AIRPORT_PREVIEW_METROPOLITAN     = SPR_AIRPORT_PREVIEW_BASE + 3;
+static const SpriteID SPR_AIRPORT_PREVIEW_INTERNATIONAL    = SPR_AIRPORT_PREVIEW_BASE + 4;
+static const SpriteID SPR_AIRPORT_PREVIEW_COMMUTER         = SPR_AIRPORT_PREVIEW_BASE + 5;
+static const SpriteID SPR_AIRPORT_PREVIEW_HELIDEPOT        = SPR_AIRPORT_PREVIEW_BASE + 6;
+static const SpriteID SPR_AIRPORT_PREVIEW_INTERCONTINENTAL = SPR_AIRPORT_PREVIEW_BASE + 7;
+static const SpriteID SPR_AIRPORT_PREVIEW_HELISTATION      = SPR_AIRPORT_PREVIEW_BASE + 8;
+static const SpriteID SPR_AIRPORT_PREVIEW_COUNT            = 9;
+
+static const SpriteID SPR_ROADSTOP_BASE     = SPR_AIRPORT_PREVIEW_BASE + SPR_AIRPORT_PREVIEW_COUNT; // The sprites used for drive-through road stops
 static const SpriteID SPR_BUS_STOP_DT_Y_W   = SPR_ROADSTOP_BASE;
 static const SpriteID SPR_BUS_STOP_DT_Y_E   = SPR_ROADSTOP_BASE + 1;
 static const SpriteID SPR_BUS_STOP_DT_X_W   = SPR_ROADSTOP_BASE + 2;
@@ -245,7 +268,7 @@ static const uint16 ONEWAY_SPRITE_COUNT = 6;
 static const SpriteID SPR_FLAGS_BASE = SPR_ONEWAY_BASE + ONEWAY_SPRITE_COUNT;
 static const uint16 FLAGS_SPRITE_COUNT = 36;
 
-/* Not really a sprite, but an empty bounding box. Used to construct bounding boxes, that help sorting the sprites, but do not have a sprite associated. */
+/* Not really a sprite, but an empty bounding box. Used to construct bounding boxes that help sorting the sprites, but do not have a sprite associated. */
 static const SpriteID SPR_EMPTY_BOUNDING_BOX = SPR_FLAGS_BASE + FLAGS_SPRITE_COUNT;
 static const uint16 EMPTY_BOUNDING_BOX_SPRITE_COUNT = 1;
 
@@ -264,9 +287,9 @@ static const SpriteID SPR_FOUNDATION_BASE = 989;
 /* Shadow cell */
 static const SpriteID SPR_SHADOW_CELL = 1004;
 
-/* Unmovables spritenumbers */
-static const SpriteID SPR_UNMOVABLE_TRANSMITTER   = 2601;
-static const SpriteID SPR_UNMOVABLE_LIGHTHOUSE    = 2602;
+/* Objects spritenumbers */
+static const SpriteID SPR_TRANSMITTER             = 2601;
+static const SpriteID SPR_LIGHTHOUSE              = 2602;
 static const SpriteID SPR_TINYHQ_NORTH            = 2603;
 static const SpriteID SPR_TINYHQ_EAST             = 2604;
 static const SpriteID SPR_TINYHQ_WEST             = 2605;
@@ -569,38 +592,38 @@ static const SpriteID SPR_WATER_SLOPE_Y_UP        = SPR_CANALS_BASE + 0; // Wate
 static const SpriteID SPR_WATER_SLOPE_X_DOWN      = SPR_CANALS_BASE + 1; // positive X
 static const SpriteID SPR_WATER_SLOPE_X_UP        = SPR_CANALS_BASE + 2; // negative X
 static const SpriteID SPR_WATER_SLOPE_Y_DOWN      = SPR_CANALS_BASE + 3; // positive Y
-/* sprites for the shiplifts
- * there are 4 kinds of shiplifts, each of them is 3 tiles long.
+/* sprites for the locks
+ * there are 4 kinds of locks, each of them is 3 tiles long.
  * the four kinds are running in the X and Y direction and
  * are "lowering" either in the "+" or the "-" direction.
  * the three tiles are the center tile (where the slope is)
  * and a bottom and a top tile */
-static const SpriteID SPR_SHIPLIFT_BASE                 = SPR_CANALS_BASE +  4;
-static const SpriteID SPR_SHIPLIFT_Y_UP_CENTER_REAR     = SPR_CANALS_BASE +  4;
-static const SpriteID SPR_SHIPLIFT_X_DOWN_CENTER_REAR   = SPR_CANALS_BASE +  5;
-static const SpriteID SPR_SHIPLIFT_X_UP_CENTER_REAR     = SPR_CANALS_BASE +  6;
-static const SpriteID SPR_SHIPLIFT_Y_DOWN_CENTER_REAR   = SPR_CANALS_BASE +  7;
-static const SpriteID SPR_SHIPLIFT_Y_UP_CENTER_FRONT    = SPR_CANALS_BASE +  8;
-static const SpriteID SPR_SHIPLIFT_X_DOWN_CENTER_FRONT  = SPR_CANALS_BASE +  9;
-static const SpriteID SPR_SHIPLIFT_X_UP_CENTER_FRONT    = SPR_CANALS_BASE + 10;
-static const SpriteID SPR_SHIPLIFT_Y_DOWN_CENTER_FRONT  = SPR_CANALS_BASE + 11;
-static const SpriteID SPR_SHIPLIFT_Y_UP_BOTTOM_REAR     = SPR_CANALS_BASE + 12;
-static const SpriteID SPR_SHIPLIFT_X_DOWN_BOTTOM_REAR   = SPR_CANALS_BASE + 13;
-static const SpriteID SPR_SHIPLIFT_X_UP_BOTTOM_REAR     = SPR_CANALS_BASE + 14;
-static const SpriteID SPR_SHIPLIFT_Y_DOWN_BOTTOM_REAR   = SPR_CANALS_BASE + 15;
-static const SpriteID SPR_SHIPLIFT_Y_UP_BOTTOM_FRONT    = SPR_CANALS_BASE + 16;
-static const SpriteID SPR_SHIPLIFT_X_DOWN_BOTTOM_FRONT  = SPR_CANALS_BASE + 17;
-static const SpriteID SPR_SHIPLIFT_X_UP_BOTTOM_FRONT    = SPR_CANALS_BASE + 18;
-static const SpriteID SPR_SHIPLIFT_Y_DOWN_BOTTOM_FRONT  = SPR_CANALS_BASE + 19;
-static const SpriteID SPR_SHIPLIFT_Y_UP_TOP_REAR        = SPR_CANALS_BASE + 20;
-static const SpriteID SPR_SHIPLIFT_X_DOWN_TOP_REAR      = SPR_CANALS_BASE + 21;
-static const SpriteID SPR_SHIPLIFT_X_UP_TOP_REAR        = SPR_CANALS_BASE + 22;
-static const SpriteID SPR_SHIPLIFT_Y_DOWN_TOP_REAR      = SPR_CANALS_BASE + 23;
-static const SpriteID SPR_SHIPLIFT_Y_UP_TOP_FRONT       = SPR_CANALS_BASE + 24;
-static const SpriteID SPR_SHIPLIFT_X_DOWN_TOP_FRONT     = SPR_CANALS_BASE + 25;
-static const SpriteID SPR_SHIPLIFT_X_UP_TOP_FRONT       = SPR_CANALS_BASE + 26;
-static const SpriteID SPR_SHIPLIFT_Y_DOWN_TOP_FRONT     = SPR_CANALS_BASE + 27;
-static const SpriteID SPR_CANAL_DIKES_BASE              = SPR_CANALS_BASE + 52;
+static const SpriteID SPR_LOCK_BASE                 = SPR_CANALS_BASE +  4;
+static const SpriteID SPR_LOCK_Y_UP_CENTER_REAR     = SPR_CANALS_BASE +  4;
+static const SpriteID SPR_LOCK_X_DOWN_CENTER_REAR   = SPR_CANALS_BASE +  5;
+static const SpriteID SPR_LOCK_X_UP_CENTER_REAR     = SPR_CANALS_BASE +  6;
+static const SpriteID SPR_LOCK_Y_DOWN_CENTER_REAR   = SPR_CANALS_BASE +  7;
+static const SpriteID SPR_LOCK_Y_UP_CENTER_FRONT    = SPR_CANALS_BASE +  8;
+static const SpriteID SPR_LOCK_X_DOWN_CENTER_FRONT  = SPR_CANALS_BASE +  9;
+static const SpriteID SPR_LOCK_X_UP_CENTER_FRONT    = SPR_CANALS_BASE + 10;
+static const SpriteID SPR_LOCK_Y_DOWN_CENTER_FRONT  = SPR_CANALS_BASE + 11;
+static const SpriteID SPR_LOCK_Y_UP_BOTTOM_REAR     = SPR_CANALS_BASE + 12;
+static const SpriteID SPR_LOCK_X_DOWN_BOTTOM_REAR   = SPR_CANALS_BASE + 13;
+static const SpriteID SPR_LOCK_X_UP_BOTTOM_REAR     = SPR_CANALS_BASE + 14;
+static const SpriteID SPR_LOCK_Y_DOWN_BOTTOM_REAR   = SPR_CANALS_BASE + 15;
+static const SpriteID SPR_LOCK_Y_UP_BOTTOM_FRONT    = SPR_CANALS_BASE + 16;
+static const SpriteID SPR_LOCK_X_DOWN_BOTTOM_FRONT  = SPR_CANALS_BASE + 17;
+static const SpriteID SPR_LOCK_X_UP_BOTTOM_FRONT    = SPR_CANALS_BASE + 18;
+static const SpriteID SPR_LOCK_Y_DOWN_BOTTOM_FRONT  = SPR_CANALS_BASE + 19;
+static const SpriteID SPR_LOCK_Y_UP_TOP_REAR        = SPR_CANALS_BASE + 20;
+static const SpriteID SPR_LOCK_X_DOWN_TOP_REAR      = SPR_CANALS_BASE + 21;
+static const SpriteID SPR_LOCK_X_UP_TOP_REAR        = SPR_CANALS_BASE + 22;
+static const SpriteID SPR_LOCK_Y_DOWN_TOP_REAR      = SPR_CANALS_BASE + 23;
+static const SpriteID SPR_LOCK_Y_UP_TOP_FRONT       = SPR_CANALS_BASE + 24;
+static const SpriteID SPR_LOCK_X_DOWN_TOP_FRONT     = SPR_CANALS_BASE + 25;
+static const SpriteID SPR_LOCK_X_UP_TOP_FRONT       = SPR_CANALS_BASE + 26;
+static const SpriteID SPR_LOCK_Y_DOWN_TOP_FRONT     = SPR_CANALS_BASE + 27;
+static const SpriteID SPR_CANAL_DIKES_BASE          = SPR_CANALS_BASE + 52;
 
 /* Sprites for tunnels and bridges */
 static const SpriteID SPR_TUNNEL_ENTRY_REAR_RAIL   = 2365;
@@ -963,6 +986,10 @@ static const SpriteID SPR_CNST1_TOWNHOUSE_06_V1               = 1444;
 static const SpriteID SPR_CNST2_TOWNHOUSE_06_V1               = 1445;
 static const SpriteID SPR_BUILD_TOWNHOUSE_06_V1               = 1446; // 1st variation
 static const SpriteID SPR_GRND_TOWNHOUSE_06_V1                = 1447;
+static const SpriteID SPR_GRND_STADIUM_N                      = 1479; //stadium ground at north
+static const SpriteID SPR_GRND_STADIUM_E                      = 1480; //stadium ground at east
+static const SpriteID SPR_GRND_STADIUM_W                      = 1481; //stadium ground at west
+static const SpriteID SPR_GRND_STADIUM_S                      = 1482; //stadium ground at south
 static const SpriteID SPR_CNST1_TOWNHOUSE_06_V2               = 1501; // used as ground, but is stage1
 static const SpriteID SPR_CNST1_TOWNHOUSE_06_V2_P             = 1502; // pipes extensions for previous
 static const SpriteID SPR_CNST2_TOWNHOUSE_06_V2_G             = 1503; // Ground of cnst stage 2
@@ -988,6 +1015,8 @@ static const SpriteID SPR_BUILD_SHOPOFFICE_0E                 = 1463;
 static const SpriteID SPR_CNST1_SHOPOFFICE_0F                 = 1464;
 static const SpriteID SPR_CNST2_SHOPOFFICE_0F                 = 1465;
 static const SpriteID SPR_BUILD_SHOPOFFICE_0F                 = 1466;
+static const SpriteID SPR_GRND_HOUSE_TOY1                     = 4675;
+static const SpriteID SPR_GRND_HOUSE_TOY2                     = 4676;
 
 /* Easter egg/disaster sprites */
 static const SpriteID SPR_BLIMP                  = 3905; // Zeppelin
@@ -1015,7 +1044,7 @@ static const SpriteID SPR_IMG_TERRAFORM_UP    = 694;
 static const SpriteID SPR_IMG_TERRAFORM_DOWN  = 695;
 static const SpriteID SPR_IMG_DYNAMITE        = 703;
 static const SpriteID SPR_IMG_ROCKS           = 4084;
-static const SpriteID SPR_IMG_LIGHTHOUSE_DESERT = 4085; // XXX - is Desert image on the desert-climate
+static const SpriteID SPR_IMG_DESERT          = 4085;
 static const SpriteID SPR_IMG_TRANSMITTER     = 4086;
 static const SpriteID SPR_IMG_LEVEL_LAND      = SPR_OPENTTD_BASE + 91;
 static const SpriteID SPR_IMG_BUILD_CANAL     = SPR_OPENTTD_BASE + 88;
@@ -1220,7 +1249,7 @@ static const SpriteID SPR_IMG_AIRPORT       = 744;
 /* dock_gui.c */
 static const SpriteID SPR_IMG_SHIP_DEPOT    = 748;
 static const SpriteID SPR_IMG_SHIP_DOCK     = 746;
-static const SpriteID SPR_IMG_BOUY          = 693;
+static const SpriteID SPR_IMG_BUOY          = 693;
 static const SpriteID SPR_IMG_AQUEDUCT      = SPR_OPENTTD_BASE + 145;
 
 /* music_gui.c */
@@ -1303,7 +1332,7 @@ static const SpriteID SPR_SELECT_TOYLAND_PUSHED      = 4889;
  * Cursors */
 static const CursorID SPR_CURSOR_MOUSE          = 0;
 static const CursorID SPR_CURSOR_ZZZ            = 1;
-static const CursorID SPR_CURSOR_BOUY           = 702;
+static const CursorID SPR_CURSOR_BUOY           = 702;
 static const CursorID SPR_CURSOR_QUERY          = 719;
 static const CursorID SPR_CURSOR_HQ             = 720;
 static const CursorID SPR_CURSOR_SHIP_DEPOT     = 721;
@@ -1316,7 +1345,7 @@ static const CursorID SPR_CURSOR_LEVEL_LAND     = SPR_OPENTTD_BASE + 92;
 static const CursorID SPR_CURSOR_TOWN           = 4080;
 static const CursorID SPR_CURSOR_INDUSTRY       = 4081;
 static const CursorID SPR_CURSOR_ROCKY_AREA     = 4087;
-static const CursorID SPR_CURSOR_LIGHTHOUSE     = 4088;
+static const CursorID SPR_CURSOR_DESERT         = 4088;
 static const CursorID SPR_CURSOR_TRANSMITTER    = 4089;
 
 /* airport cursors */
@@ -1397,6 +1426,26 @@ static const CursorID SPR_CURSOR_CLONE_AIRPLANE = SPR_OPENTTD_BASE + 113;
 
 /** Animation macro in table/animcursors.h (_animcursors[]) */
 
+static const CursorID SPR_CURSOR_DEMOLISH_FIRST = 704;
+static const CursorID SPR_CURSOR_DEMOLISH_1     = 705;
+static const CursorID SPR_CURSOR_DEMOLISH_2     = 706;
+static const CursorID SPR_CURSOR_DEMOLISH_LAST  = 707;
+
+static const CursorID SPR_CURSOR_LOWERLAND_FIRST = 699;
+static const CursorID SPR_CURSOR_LOWERLAND_1     = 700;
+static const CursorID SPR_CURSOR_LOWERLAND_LAST  = 701;
+
+static const CursorID SPR_CURSOR_RAISELAND_FIRST = 696;
+static const CursorID SPR_CURSOR_RAISELAND_1     = 697;
+static const CursorID SPR_CURSOR_RAISELAND_LAST  = 698;
+
+static const CursorID SPR_CURSOR_PICKSTATION_FIRST = 716;
+static const CursorID SPR_CURSOR_PICKSTATION_1     = 717;
+static const CursorID SPR_CURSOR_PICKSTATION_LAST  = 718;
+
+static const CursorID SPR_CURSOR_BUILDSIGNALS_FIRST = 1292;
+static const CursorID SPR_CURSOR_BUILDSIGNALS_LAST  = 1293;
+
 /** Flag for saying a cursor sprite is an animated cursor. */
 static const CursorID ANIMCURSOR_FLAG         = 1U << 31;
 static const CursorID ANIMCURSOR_DEMOLISH     = ANIMCURSOR_FLAG | 0; ///<  704 -  707 - demolish dynamite
@@ -1444,9 +1493,11 @@ enum Modifiers {
 	PALETTE_MODIFIER_COLOUR       = RECOLOUR_BIT,
 };
 
-/** Masks needed for sprite operations.
+/**
+ * Masks needed for sprite operations.
  * @note Do not modify this enum. Alter SpriteSetup instead
- * @see SpriteSetup */
+ * @see SpriteSetup
+ */
 enum SpriteMasks {
 	/** Maximum number of sprites that can be loaded at a given time. */
 	MAX_SPRITES = 1 << SPRITE_WIDTH,
