@@ -1,4 +1,4 @@
-/* $Id: terraform_gui.cpp 19854 2010-05-18 21:25:03Z rubidium $ */
+/* $Id: terraform_gui.cpp 22018 2011-02-07 22:37:22Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -19,19 +19,19 @@
 #include "viewport_func.h"
 #include "command_func.h"
 #include "signs_func.h"
-#include "variables.h"
-#include "functions.h"
 #include "sound_func.h"
 #include "base_station_base.h"
-#include "unmovable_map.h"
 #include "textbuf_gui.h"
 #include "genworld.h"
 #include "tree_map.h"
 #include "landscape_type.h"
 #include "tilehighlight_func.h"
 #include "strings_func.h"
+#include "newgrf_object.h"
+#include "object.h"
+#include "hotkeys.h"
+#include "engine_base.h"
 
-#include "table/sprites.h"
 #include "table/strings.h"
 
 void CcTerraform(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2)
@@ -59,6 +59,7 @@ static void GenerateDesertArea(TileIndex end, TileIndex start)
 		MarkTileDirtyByTile(tile);
 	}
 	_generating_world = false;
+	InvalidateWindowClassesData(WC_TOWN_VIEW, 0);
 }
 
 /** Scenario editor command that generates rocky areas */
@@ -73,7 +74,7 @@ static void GenerateRockyArea(TileIndex end, TileIndex start)
 		switch (GetTileType(tile)) {
 			case MP_TREES:
 				if (GetTreeGround(tile) == TREE_GROUND_SHORE) continue;
-			/* FALL THROUGH */
+				/* FALL THROUGH */
 			case MP_CLEAR:
 				MakeClear(tile, CLEAR_ROCKS, 3);
 				break;
@@ -95,7 +96,7 @@ static void GenerateRockyArea(TileIndex end, TileIndex start)
  * @return Returns true if the action was found and handled, and false otherwise. This
  * allows for additional implements that are more local. For example X_Y drag
  * of convertrail which belongs in rail_gui.cpp and not terraform_gui.cpp
- **/
+ */
 bool GUIPlaceProcDragXY(ViewportDragDropSelectionProcess proc, TileIndex start_tile, TileIndex end_tile)
 {
 	if (!_settings_game.construction.freeform_edges) {
@@ -107,16 +108,16 @@ bool GUIPlaceProcDragXY(ViewportDragDropSelectionProcess proc, TileIndex start_t
 
 	switch (proc) {
 		case DDSP_DEMOLISH_AREA:
-			DoCommandP(end_tile, start_tile, 0, CMD_CLEAR_AREA | CMD_MSG(STR_ERROR_CAN_T_CLEAR_THIS_AREA), CcPlaySound10);
+			DoCommandP(end_tile, start_tile, _ctrl_pressed ? 1 : 0, CMD_CLEAR_AREA | CMD_MSG(STR_ERROR_CAN_T_CLEAR_THIS_AREA), CcPlaySound10);
 			break;
 		case DDSP_RAISE_AND_LEVEL_AREA:
-			DoCommandP(end_tile, start_tile, 1, CMD_LEVEL_LAND | CMD_MSG(STR_ERROR_CAN_T_RAISE_LAND_HERE), CcTerraform);
+			DoCommandP(end_tile, start_tile, LM_RAISE << 1 | (_ctrl_pressed ? 1 : 0), CMD_LEVEL_LAND | CMD_MSG(STR_ERROR_CAN_T_RAISE_LAND_HERE), CcTerraform);
 			break;
 		case DDSP_LOWER_AND_LEVEL_AREA:
-			DoCommandP(end_tile, start_tile, (uint32)-1, CMD_LEVEL_LAND | CMD_MSG(STR_ERROR_CAN_T_LOWER_LAND_HERE), CcTerraform);
+			DoCommandP(end_tile, start_tile, LM_LOWER << 1 | (_ctrl_pressed ? 1 : 0), CMD_LEVEL_LAND | CMD_MSG(STR_ERROR_CAN_T_LOWER_LAND_HERE), CcTerraform);
 			break;
 		case DDSP_LEVEL_AREA:
-			DoCommandP(end_tile, start_tile, 0, CMD_LEVEL_LAND | CMD_MSG(STR_ERROR_CAN_T_LEVEL_LAND_HERE), CcTerraform);
+			DoCommandP(end_tile, start_tile, LM_LEVEL << 1 | (_ctrl_pressed ? 1 : 0), CMD_LEVEL_LAND | CMD_MSG(STR_ERROR_CAN_T_LEVEL_LAND_HERE), CcTerraform);
 			break;
 		case DDSP_CREATE_ROCKS:
 			GenerateRockyArea(end_tile, start_tile);
@@ -131,45 +132,18 @@ bool GUIPlaceProcDragXY(ViewportDragDropSelectionProcess proc, TileIndex start_t
 	return true;
 }
 
-typedef void OnButtonClick(Window *w);
-
-static const uint16 _terraform_keycodes[] = {
-	'Q',
-	'W',
-	'E',
-	'D',
-	'U',
-	'I',
-	'O',
-};
-
-static void PlaceProc_BuyLand(TileIndex tile)
-{
-	DoCommandP(tile, 0, 0, CMD_PURCHASE_LAND_AREA | CMD_MSG(STR_ERROR_CAN_T_PURCHASE_THIS_LAND), CcPlaySound1E);
-}
-
+/**
+ * Start a drag for demolishing an area.
+ * @param tile Position of one corner.
+ */
 void PlaceProc_DemolishArea(TileIndex tile)
 {
 	VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_DEMOLISH_AREA);
 }
 
-static void PlaceProc_RaiseLand(TileIndex tile)
-{
-	VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_RAISE_AND_LEVEL_AREA);
-}
-
-static void PlaceProc_LowerLand(TileIndex tile)
-{
-	VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_LOWER_AND_LEVEL_AREA);
-}
-
-static void PlaceProc_LevelLand(TileIndex tile)
-{
-	VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_LEVEL_AREA);
-}
-
 /** Enum referring to the widgets of the terraform toolbar */
 enum TerraformToolbarWidgets {
+	TTW_SHOW_PLACE_OBJECT,                ///< Should the place object button be shown?
 	TTW_BUTTONS_START,                    ///< Start of pushable buttons
 	TTW_LOWER_LAND = TTW_BUTTONS_START,   ///< Lower land button
 	TTW_RAISE_LAND,                       ///< Raise land button
@@ -178,88 +152,133 @@ enum TerraformToolbarWidgets {
 	TTW_BUY_LAND,                         ///< Buy land button
 	TTW_PLANT_TREES,                      ///< Plant trees button (note: opens seperate window, no place-push-button)
 	TTW_PLACE_SIGN,                       ///< Place sign button
+	TTW_PLACE_OBJECT,                     ///< Place object button
 };
 
-static void TerraformClick_Lower(Window *w)
-{
-	HandlePlacePushButton(w, TTW_LOWER_LAND, ANIMCURSOR_LOWERLAND, HT_POINT, PlaceProc_LowerLand);
-}
-
-static void TerraformClick_Raise(Window *w)
-{
-	HandlePlacePushButton(w, TTW_RAISE_LAND, ANIMCURSOR_RAISELAND, HT_POINT, PlaceProc_RaiseLand);
-}
-
-static void TerraformClick_Level(Window *w)
-{
-	HandlePlacePushButton(w, TTW_LEVEL_LAND, SPR_CURSOR_LEVEL_LAND, HT_POINT, PlaceProc_LevelLand);
-}
-
-static void TerraformClick_Dynamite(Window *w)
-{
-	HandlePlacePushButton(w, TTW_DEMOLISH, ANIMCURSOR_DEMOLISH, HT_RECT, PlaceProc_DemolishArea);
-}
-
-static void TerraformClick_BuyLand(Window *w)
-{
-	HandlePlacePushButton(w, TTW_BUY_LAND, SPR_CURSOR_BUY_LAND, HT_RECT, PlaceProc_BuyLand);
-}
-
-static void TerraformClick_Trees(Window *w)
-{
-	/* This button is NOT a place-push-button, so don't treat it as such */
-	ShowBuildTreesToolbar();
-}
-
-static void TerraformClick_PlaceSign(Window *w)
-{
-	HandlePlacePushButton(w, TTW_PLACE_SIGN, SPR_CURSOR_SIGN, HT_RECT, PlaceProc_Sign);
-}
-
-static OnButtonClick * const _terraform_button_proc[] = {
-	TerraformClick_Lower,
-	TerraformClick_Raise,
-	TerraformClick_Level,
-	TerraformClick_Dynamite,
-	TerraformClick_BuyLand,
-	TerraformClick_Trees,
-	TerraformClick_PlaceSign,
-};
-
+/** Terra form toolbar managing class. */
 struct TerraformToolbarWindow : Window {
+	int last_user_action; ///< Last started user action.
+
 	TerraformToolbarWindow(const WindowDesc *desc, WindowNumber window_number) : Window()
 	{
-		this->InitNested(desc, window_number);
+		/* This is needed as we like to have the tree available on OnInit. */
+		this->CreateNestedTree(desc);
+		this->FinishInitNested(desc, window_number);
+		this->last_user_action = WIDGET_LIST_END;
 	}
 
 	~TerraformToolbarWindow()
 	{
 	}
 
-	virtual void OnPaint()
+	virtual void OnInit()
 	{
-		this->DrawWidgets();
+		/* Don't show the place object button when there are no objects to place. */
+		NWidgetStacked *show_object = this->GetWidget<NWidgetStacked>(TTW_SHOW_PLACE_OBJECT);
+		show_object->SetDisplayedPlane(ObjectClass::GetCount() != 0 ? 0 : SZSP_NONE);
 	}
 
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
-		if (widget >= TTW_BUTTONS_START) _terraform_button_proc[widget - TTW_BUTTONS_START](this);
+		if (widget < TTW_BUTTONS_START) return;
+
+		switch (widget) {
+			case TTW_LOWER_LAND: // Lower land button
+				HandlePlacePushButton(this, TTW_LOWER_LAND, ANIMCURSOR_LOWERLAND, HT_POINT | HT_DIAGONAL);
+				this->last_user_action = widget;
+				break;
+
+			case TTW_RAISE_LAND: // Raise land button
+				HandlePlacePushButton(this, TTW_RAISE_LAND, ANIMCURSOR_RAISELAND, HT_POINT | HT_DIAGONAL);
+				this->last_user_action = widget;
+				break;
+
+			case TTW_LEVEL_LAND: // Level land button
+				HandlePlacePushButton(this, TTW_LEVEL_LAND, SPR_CURSOR_LEVEL_LAND, HT_POINT | HT_DIAGONAL);
+				this->last_user_action = widget;
+				break;
+
+			case TTW_DEMOLISH: // Demolish aka dynamite button
+				HandlePlacePushButton(this, TTW_DEMOLISH, ANIMCURSOR_DEMOLISH, HT_RECT | HT_DIAGONAL);
+				this->last_user_action = widget;
+				break;
+
+			case TTW_BUY_LAND: // Buy land button
+				HandlePlacePushButton(this, TTW_BUY_LAND, SPR_CURSOR_BUY_LAND, HT_RECT);
+				this->last_user_action = widget;
+				break;
+
+			case TTW_PLANT_TREES: // Plant trees button
+				/* This button is NOT a place-push-button, so don't treat it as such */
+				this->HandleButtonClick(TTW_PLANT_TREES);
+				ShowBuildTreesToolbar();
+				break;
+
+			case TTW_PLACE_SIGN: // Place sign button
+				HandlePlacePushButton(this, TTW_PLACE_SIGN, SPR_CURSOR_SIGN, HT_RECT);
+				this->last_user_action = widget;
+				break;
+
+			case TTW_PLACE_OBJECT: // Place object button
+				/* Don't show the place object button when there are no objects to place. */
+				if (ObjectClass::GetCount() == 0) return;
+				if (HandlePlacePushButton(this, TTW_PLACE_OBJECT, SPR_CURSOR_TRANSMITTER, HT_RECT)) {
+					ShowBuildObjectPicker(this);
+					this->last_user_action = widget;
+				}
+				break;
+
+			default: NOT_REACHED();
+		}
+	}
+
+	virtual void OnTimeout()
+	{
+		this->RaiseWidget(TTW_PLANT_TREES);
+		this->SetWidgetDirty(TTW_PLANT_TREES);
 	}
 
 	virtual EventState OnKeyPress(uint16 key, uint16 keycode)
 	{
-		for (uint i = 0; i != lengthof(_terraform_keycodes); i++) {
-			if (keycode == _terraform_keycodes[i]) {
-				_terraform_button_proc[i](this);
-				return ES_HANDLED;
-			}
-		}
-		return ES_NOT_HANDLED;
+		int num = CheckHotkeyMatch(terraform_hotkeys, keycode, this);
+		if (num == -1) return ES_NOT_HANDLED;
+		this->OnClick(Point(), num, 1);
+		return ES_HANDLED;
 	}
 
 	virtual void OnPlaceObject(Point pt, TileIndex tile)
 	{
-		_place_proc(tile);
+		switch (this->last_user_action) {
+			case TTW_LOWER_LAND: // Lower land button
+				VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_LOWER_AND_LEVEL_AREA);
+				break;
+
+			case TTW_RAISE_LAND: // Raise land button
+				VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_RAISE_AND_LEVEL_AREA);
+				break;
+
+			case TTW_LEVEL_LAND: // Level land button
+				VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_LEVEL_AREA);
+				break;
+
+			case TTW_DEMOLISH: // Demolish aka dynamite button
+				PlaceProc_DemolishArea(tile);
+				break;
+
+			case TTW_BUY_LAND: // Buy land button
+				DoCommandP(tile, OBJECT_OWNED_LAND, 0, CMD_BUILD_OBJECT | CMD_MSG(STR_ERROR_CAN_T_PURCHASE_THIS_LAND), CcPlaySound1E);
+				break;
+
+			case TTW_PLACE_SIGN: // Place sign button
+				PlaceProc_Sign(tile);
+				break;
+
+			case TTW_PLACE_OBJECT: // Place object button
+				PlaceProc_Object(tile);
+				break;
+
+			default: NOT_REACHED();
+		}
 	}
 
 	virtual void OnPlaceDrag(ViewportPlaceMethod select_method, ViewportDragDropSelectionProcess select_proc, Point pt)
@@ -291,10 +310,26 @@ struct TerraformToolbarWindow : Window {
 
 	virtual void OnPlaceObjectAbort()
 	{
+		DeleteWindowById(WC_BUILD_OBJECT, 0);
 		this->RaiseButtons();
 	}
+
+	static Hotkey<TerraformToolbarWindow> terraform_hotkeys[];
 };
 
+Hotkey<TerraformToolbarWindow> TerraformToolbarWindow::terraform_hotkeys[] = {
+	Hotkey<TerraformToolbarWindow>('Q' | WKC_GLOBAL_HOTKEY, "lower", TTW_LOWER_LAND),
+	Hotkey<TerraformToolbarWindow>('W' | WKC_GLOBAL_HOTKEY, "raise", TTW_RAISE_LAND),
+	Hotkey<TerraformToolbarWindow>('E' | WKC_GLOBAL_HOTKEY, "level", TTW_LEVEL_LAND),
+	Hotkey<TerraformToolbarWindow>('D' | WKC_GLOBAL_HOTKEY, "dynamite", TTW_DEMOLISH),
+	Hotkey<TerraformToolbarWindow>('U', "buyland", TTW_BUY_LAND),
+	Hotkey<TerraformToolbarWindow>('I', "trees", TTW_PLANT_TREES),
+	Hotkey<TerraformToolbarWindow>('O', "placesign", TTW_PLACE_SIGN),
+	Hotkey<TerraformToolbarWindow>('P', "placeobject", TTW_PLACE_OBJECT),
+	HOTKEY_LIST_END(TerraformToolbarWindow)
+};
+Hotkey<TerraformToolbarWindow> *_terraform_hotkeys = TerraformToolbarWindow::terraform_hotkeys;
+
 static const NWidgetPart _nested_terraform_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_DARK_GREEN),
@@ -302,23 +337,27 @@ static const NWidgetPart _nested_terraform_widgets[] = {
 		NWidget(WWT_STICKYBOX, COLOUR_DARK_GREEN),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_LOWER_LAND), SetMinimalSize(22,22),
+		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_LOWER_LAND), SetMinimalSize(22, 22),
 								SetFill(0, 1), SetDataTip(SPR_IMG_TERRAFORM_DOWN, STR_LANDSCAPING_TOOLTIP_LOWER_A_CORNER_OF_LAND),
-		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_RAISE_LAND), SetMinimalSize(22,22),
+		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_RAISE_LAND), SetMinimalSize(22, 22),
 								SetFill(0, 1), SetDataTip(SPR_IMG_TERRAFORM_UP, STR_LANDSCAPING_TOOLTIP_RAISE_A_CORNER_OF_LAND),
-		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_LEVEL_LAND), SetMinimalSize(22,22),
+		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_LEVEL_LAND), SetMinimalSize(22, 22),
 								SetFill(0, 1), SetDataTip(SPR_IMG_LEVEL_LAND, STR_LANDSCAPING_LEVEL_LAND_TOOLTIP),
 
 		NWidget(WWT_PANEL, COLOUR_DARK_GREEN), SetMinimalSize(4, 22), EndContainer(),
 
-		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_DEMOLISH), SetMinimalSize(22,22),
+		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_DEMOLISH), SetMinimalSize(22, 22),
 								SetFill(0, 1), SetDataTip(SPR_IMG_DYNAMITE, STR_TOOLTIP_DEMOLISH_BUILDINGS_ETC),
-		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_BUY_LAND), SetMinimalSize(22,22),
+		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_BUY_LAND), SetMinimalSize(22, 22),
 								SetFill(0, 1), SetDataTip(SPR_IMG_BUY_LAND, STR_LANDSCAPING_TOOLTIP_PURCHASE_LAND),
-		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_PLANT_TREES), SetMinimalSize(22,22),
+		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_PLANT_TREES), SetMinimalSize(22, 22),
 								SetFill(0, 1), SetDataTip(SPR_IMG_PLANTTREES, STR_SCENEDIT_TOOLBAR_PLANT_TREES),
-		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_PLACE_SIGN), SetMinimalSize(22,22),
+		NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_PLACE_SIGN), SetMinimalSize(22, 22),
 								SetFill(0, 1), SetDataTip(SPR_IMG_SIGN, STR_SCENEDIT_TOOLBAR_PLACE_SIGN),
+		NWidget(NWID_SELECTION, INVALID_COLOUR, TTW_SHOW_PLACE_OBJECT),
+			NWidget(WWT_IMGBTN, COLOUR_DARK_GREEN, TTW_PLACE_OBJECT), SetMinimalSize(22, 22),
+								SetFill(0, 1), SetDataTip(SPR_IMG_TRANSMITTER, STR_SCENEDIT_TOOLBAR_PLACE_OBJECT),
+		EndContainer(),
 	EndContainer(),
 };
 
@@ -346,21 +385,20 @@ Window *ShowTerraformToolbar(Window *link)
 	w->top -= w->height;
 	w->SetDirty();
 	/* Put the linked toolbar to the left / right of it. */
-	link->left = w->left + (_dynlang.text_dir == TD_RTL ? w->width : -link->width);
+	link->left = w->left + (_current_text_dir == TD_RTL ? w->width : -link->width);
 	link->top  = w->top;
 	link->SetDirty();
 
 	return w;
 }
 
-void ShowTerraformToolbarWithTool(uint16 key, uint16 keycode)
+EventState TerraformToolbarGlobalHotkeys(uint16 key, uint16 keycode)
 {
-	Window *w = FindWindowById(WC_SCEN_LAND_GEN, 0);
-
-	if (w == NULL) w = ShowTerraformToolbar(NULL);
-	if (w == NULL) return;
-
-	w->OnKeyPress(key, keycode);
+	int num = CheckHotkeyMatch<TerraformToolbarWindow>(_terraform_hotkeys, keycode, NULL, true);
+	if (num == -1) return ES_NOT_HANDLED;
+	Window *w = ShowTerraformToolbar(NULL);
+	if (w == NULL) return ES_NOT_HANDLED;
+	return w->OnKeyPress(key, keycode);
 }
 
 static byte _terraform_size = 1;
@@ -376,9 +414,6 @@ static byte _terraform_size = 1;
  */
 static void CommonRaiseLowerBigLand(TileIndex tile, int mode)
 {
-	int sizex, sizey;
-	uint h;
-
 	if (_terraform_size == 1) {
 		StringID msg =
 			mode ? STR_ERROR_CAN_T_RAISE_LAND_HERE : STR_ERROR_CAN_T_LOWER_LAND_HERE;
@@ -386,29 +421,29 @@ static void CommonRaiseLowerBigLand(TileIndex tile, int mode)
 		DoCommandP(tile, SLOPE_N, (uint32)mode, CMD_TERRAFORM_LAND | CMD_MSG(msg), CcTerraform);
 	} else {
 		assert(_terraform_size != 0);
-		/* check out for map overflows */
-		sizex = min(MapSizeX() - TileX(tile), _terraform_size);
-		sizey = min(MapSizeY() - TileY(tile), _terraform_size);
+		TileArea ta(tile, _terraform_size, _terraform_size);
+		ta.ClampToMap();
 
-		if (sizex == 0 || sizey == 0) return;
+		if (ta.w == 0 || ta.h == 0) return;
 
 		SndPlayTileFx(SND_1F_SPLAT, tile);
 
+		uint h;
 		if (mode != 0) {
 			/* Raise land */
-			h = 15; // XXX - max height
-			TILE_LOOP(tile2, sizex, sizey, tile) {
+			h = MAX_TILE_HEIGHT;
+			TILE_AREA_LOOP(tile2, ta) {
 				h = min(h, TileHeight(tile2));
 			}
 		} else {
 			/* Lower land */
 			h = 0;
-			TILE_LOOP(tile2, sizex, sizey, tile) {
+			TILE_AREA_LOOP(tile2, ta) {
 				h = max(h, TileHeight(tile2));
 			}
 		}
 
-		TILE_LOOP(tile2, sizex, sizey, tile) {
+		TILE_AREA_LOOP(tile2, ta) {
 			if (TileHeight(tile2) == h) {
 				DoCommandP(tile2, SLOPE_N, (uint32)mode, CMD_TERRAFORM_LAND);
 			}
@@ -416,50 +451,6 @@ static void CommonRaiseLowerBigLand(TileIndex tile, int mode)
 	}
 }
 
-static void PlaceProc_RaiseBigLand(TileIndex tile)
-{
-	CommonRaiseLowerBigLand(tile, 1);
-}
-
-static void PlaceProc_LowerBigLand(TileIndex tile)
-{
-	CommonRaiseLowerBigLand(tile, 0);
-}
-
-static void PlaceProc_RockyArea(TileIndex tile)
-{
-	VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_CREATE_ROCKS);
-}
-
-static void PlaceProc_LightHouse(TileIndex tile)
-{
-	/* not flat || not(trees || clear without bridge above) */
-	if (GetTileSlope(tile, NULL) != SLOPE_FLAT || !(IsTileType(tile, MP_TREES) || (IsTileType(tile, MP_CLEAR) && !IsBridgeAbove(tile)))) {
-		return;
-	}
-
-	MakeLighthouse(tile);
-	MarkTileDirtyByTile(tile);
-	SndPlayTileFx(SND_1F_SPLAT, tile);
-}
-
-static void PlaceProc_Transmitter(TileIndex tile)
-{
-	/* not flat || not(trees || clear without bridge above) */
-	if (GetTileSlope(tile, NULL) != SLOPE_FLAT || !(IsTileType(tile, MP_TREES) || (IsTileType(tile, MP_CLEAR) && !IsBridgeAbove(tile)))) {
-		return;
-	}
-
-	MakeTransmitter(tile);
-	MarkTileDirtyByTile(tile);
-	SndPlayTileFx(SND_1F_SPLAT, tile);
-}
-
-static void PlaceProc_DesertArea(TileIndex tile)
-{
-	VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_CREATE_DESERT);
-}
-
 static const int8 _multi_terraform_coords[][2] = {
 	{  0, -2},
 	{  4,  0}, { -4,  0}, {  0,  2},
@@ -473,7 +464,8 @@ static const int8 _multi_terraform_coords[][2] = {
 
 /** Enum referring to the widgets of the editor terraform toolbar */
 enum EditorTerraformToolbarWidgets {
-	ETTW_START = 0,                        ///< Used for iterations
+	ETTW_SHOW_PLACE_DESERT,                ///< Should the place desert button be shown?
+	ETTW_START,                            ///< Used for iterations
 	ETTW_DOTS = ETTW_START,                ///< Invisible widget for rendering the terraform size on.
 	ETTW_BUTTONS_START,                    ///< Start of pushable buttons
 	ETTW_DEMOLISH = ETTW_BUTTONS_START,    ///< Demolish aka dynamite button
@@ -481,8 +473,8 @@ enum EditorTerraformToolbarWidgets {
 	ETTW_RAISE_LAND,                       ///< Raise land button
 	ETTW_LEVEL_LAND,                       ///< Level land button
 	ETTW_PLACE_ROCKS,                      ///< Place rocks button
-	ETTW_PLACE_DESERT_LIGHTHOUSE,          ///< Place desert button (in tropical climate) / place lighthouse button (else)
-	ETTW_PLACE_TRANSMITTER,                ///< Place transmitter button
+	ETTW_PLACE_DESERT,                     ///< Place desert button (in tropical climate)
+	ETTW_PLACE_OBJECT,                     ///< Place transmitter button
 	ETTW_BUTTONS_END,                      ///< End of pushable buttons
 	ETTW_INCREASE_SIZE = ETTW_BUTTONS_END, ///< Upwards arrow button to increase terraforming size
 	ETTW_DECREASE_SIZE,                    ///< Downwards arrow button to decrease terraforming size
@@ -510,10 +502,12 @@ static const NWidgetPart _nested_scen_edit_land_gen_widgets[] = {
 										SetFill(0, 1), SetDataTip(SPR_IMG_LEVEL_LAND, STR_LANDSCAPING_LEVEL_LAND_TOOLTIP),
 			NWidget(WWT_IMGBTN, COLOUR_GREY, ETTW_PLACE_ROCKS), SetMinimalSize(22, 22),
 										SetFill(0, 1), SetDataTip(SPR_IMG_ROCKS, STR_TERRAFORM_TOOLTIP_PLACE_ROCKY_AREAS_ON_LANDSCAPE),
-			NWidget(WWT_IMGBTN, COLOUR_GREY, ETTW_PLACE_DESERT_LIGHTHOUSE), SetMinimalSize(22, 22),
-										SetFill(0, 1), SetDataTip(SPR_IMG_LIGHTHOUSE_DESERT, STR_NULL),
-			NWidget(WWT_IMGBTN, COLOUR_GREY, ETTW_PLACE_TRANSMITTER), SetMinimalSize(23, 22),
-										SetFill(0, 1), SetDataTip(SPR_IMG_TRANSMITTER, STR_TERRAFORM_TOOLTIP_PLACE_TRANSMITTER),
+			NWidget(NWID_SELECTION, INVALID_COLOUR, ETTW_SHOW_PLACE_DESERT),
+				NWidget(WWT_IMGBTN, COLOUR_GREY, ETTW_PLACE_DESERT), SetMinimalSize(22, 22),
+											SetFill(0, 1), SetDataTip(SPR_IMG_DESERT, STR_TERRAFORM_TOOLTIP_DEFINE_DESERT_AREA),
+			EndContainer(),
+			NWidget(WWT_IMGBTN, COLOUR_GREY, ETTW_PLACE_OBJECT), SetMinimalSize(23, 22),
+										SetFill(0, 1), SetDataTip(SPR_IMG_TRANSMITTER, STR_SCENEDIT_TOOLBAR_PLACE_OBJECT),
 			NWidget(NWID_SPACER), SetFill(1, 0),
 		EndContainer(),
 		NWidget(NWID_HORIZONTAL),
@@ -538,69 +532,10 @@ static const NWidgetPart _nested_scen_edit_land_gen_widgets[] = {
 };
 
 /**
- * @todo Merge with terraform_gui.cpp (move there) after I have cooled down at its braindeadness
- * and changed OnButtonClick to include the widget as well in the function declaration. Post 0.4.0 - Darkvater
- */
-static void EditorTerraformClick_Dynamite(Window *w)
-{
-	HandlePlacePushButton(w, ETTW_DEMOLISH, ANIMCURSOR_DEMOLISH, HT_RECT, PlaceProc_DemolishArea);
-}
-
-static void EditorTerraformClick_LowerBigLand(Window *w)
-{
-	HandlePlacePushButton(w, ETTW_LOWER_LAND, ANIMCURSOR_LOWERLAND, HT_POINT, PlaceProc_LowerBigLand);
-}
-
-static void EditorTerraformClick_RaiseBigLand(Window *w)
-{
-	HandlePlacePushButton(w, ETTW_RAISE_LAND, ANIMCURSOR_RAISELAND, HT_POINT, PlaceProc_RaiseBigLand);
-}
-
-static void EditorTerraformClick_LevelLand(Window *w)
-{
-	HandlePlacePushButton(w, ETTW_LEVEL_LAND, SPR_CURSOR_LEVEL_LAND, HT_POINT, PlaceProc_LevelLand);
-}
-
-static void EditorTerraformClick_RockyArea(Window *w)
-{
-	HandlePlacePushButton(w, ETTW_PLACE_ROCKS, SPR_CURSOR_ROCKY_AREA, HT_RECT, PlaceProc_RockyArea);
-}
-
-static void EditorTerraformClick_DesertLightHouse(Window *w)
-{
-	HandlePlacePushButton(w, ETTW_PLACE_DESERT_LIGHTHOUSE, SPR_CURSOR_LIGHTHOUSE, HT_RECT, (_settings_game.game_creation.landscape == LT_TROPIC) ? PlaceProc_DesertArea : PlaceProc_LightHouse);
-}
-
-static void EditorTerraformClick_Transmitter(Window *w)
-{
-	HandlePlacePushButton(w, ETTW_PLACE_TRANSMITTER, SPR_CURSOR_TRANSMITTER, HT_RECT, PlaceProc_Transmitter);
-}
-
-static const uint16 _editor_terraform_keycodes[] = {
-	'D',
-	'Q',
-	'W',
-	'E',
-	'R',
-	'T',
-	'Y'
-};
-
-typedef void OnButtonClick(Window *w);
-static OnButtonClick * const _editor_terraform_button_proc[] = {
-	EditorTerraformClick_Dynamite,
-	EditorTerraformClick_LowerBigLand,
-	EditorTerraformClick_RaiseBigLand,
-	EditorTerraformClick_LevelLand,
-	EditorTerraformClick_RockyArea,
-	EditorTerraformClick_DesertLightHouse,
-	EditorTerraformClick_Transmitter
-};
-
-
-/** Callback function for the scenario editor 'reset landscape' confirmation window
+ * Callback function for the scenario editor 'reset landscape' confirmation window
  * @param w Window unused
- * @param confirmed boolean value, true when yes was clicked, false otherwise */
+ * @param confirmed boolean value, true when yes was clicked, false otherwise
+ */
 static void ResetLandscapeConfirmationCallback(Window *w, bool confirmed)
 {
 	if (confirmed) {
@@ -625,15 +560,24 @@ static void ResetLandscapeConfirmationCallback(Window *w, bool confirmed)
 			if (!st->IsInUse()) delete st;
 		}
 
+		/* Now that all vehicles are gone, we can reset the engine pool. Maybe it reduces some NewGRF changing-mess */
+		EngineOverrideManager::ResetToCurrentNewGRFConfig();
+
 		MarkWholeScreenDirty();
 	}
 }
 
+/** Landscape generation window handler in the scenario editor. */
 struct ScenarioEditorLandscapeGenerationWindow : Window {
+	int last_user_action; ///< Last started user action.
+
 	ScenarioEditorLandscapeGenerationWindow(const WindowDesc *desc, WindowNumber window_number) : Window()
 	{
-		this->InitNested(desc, window_number);
-		this->GetWidget<NWidgetCore>(ETTW_PLACE_DESERT_LIGHTHOUSE)->tool_tip = (_settings_game.game_creation.landscape == LT_TROPIC) ? STR_TERRAFORM_TOOLTIP_DEFINE_DESERT_AREA : STR_TERRAFORM_TOOLTIP_PLACE_LIGHTHOUSE;
+		this->CreateNestedTree(desc);
+		NWidgetStacked *show_desert = this->GetWidget<NWidgetStacked>(ETTW_SHOW_PLACE_DESERT);
+		show_desert->SetDisplayedPlane(_settings_game.game_creation.landscape == LT_TROPIC ? 0 : SZSP_NONE);
+		this->FinishInitNested(desc, window_number);
+		this->last_user_action = WIDGET_LIST_END;
 	}
 
 	virtual void OnPaint()
@@ -649,8 +593,8 @@ struct ScenarioEditorLandscapeGenerationWindow : Window {
 	{
 		if (widget != ETTW_DOTS) return;
 
-		int center_x = (r.left + r.right + 1) / 2;
-		int center_y = (r.top + r.bottom + 1) / 2;
+		int center_x = RoundDivSU(r.left + r.right, 2);
+		int center_y = RoundDivSU(r.top + r.bottom, 2);
 
 		int n = _terraform_size * _terraform_size;
 		const int8 *coords = &_multi_terraform_coords[0][0];
@@ -664,45 +608,78 @@ struct ScenarioEditorLandscapeGenerationWindow : Window {
 
 	virtual EventState OnKeyPress(uint16 key, uint16 keycode)
 	{
-		for (uint i = 0; i != lengthof(_editor_terraform_keycodes); i++) {
-			if (keycode == _editor_terraform_keycodes[i]) {
-				_editor_terraform_button_proc[i](this);
-				return ES_HANDLED;
-			}
-		}
-		return ES_NOT_HANDLED;
+		int num = CheckHotkeyMatch(terraform_editor_hotkeys, keycode, this);
+		if (num == -1) return ES_NOT_HANDLED;
+		this->OnClick(Point(), num, 1);
+		return ES_HANDLED;
 	}
 
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
-		if (IsInsideMM(widget, ETTW_BUTTONS_START, ETTW_BUTTONS_END)) {
-			_editor_terraform_button_proc[widget - ETTW_BUTTONS_START](this);
-		} else {
-			switch (widget) {
-				case ETTW_INCREASE_SIZE:
-				case ETTW_DECREASE_SIZE: { // Increase/Decrease terraform size
-					int size = (widget == ETTW_INCREASE_SIZE) ? 1 : -1;
-					this->HandleButtonClick(widget);
-					size += _terraform_size;
+		if (widget < ETTW_BUTTONS_START) return;
 
-					if (!IsInsideMM(size, 1, 8 + 1)) return;
-					_terraform_size = size;
+		switch (widget) {
+			case ETTW_DEMOLISH: // Demolish aka dynamite button
+				HandlePlacePushButton(this, ETTW_DEMOLISH, ANIMCURSOR_DEMOLISH, HT_RECT | HT_DIAGONAL);
+				this->last_user_action = widget;
+				break;
 
-					SndPlayFx(SND_15_BEEP);
-					this->SetDirty();
-				} break;
-				case ETTW_NEW_SCENARIO: // gen random land
-					this->HandleButtonClick(widget);
-					ShowCreateScenario();
-					break;
-				case ETTW_RESET_LANDSCAPE: // Reset landscape
-					ShowQuery(
-						STR_QUERY_RESET_LANDSCAPE_CAPTION,
-						STR_RESET_LANDSCAPE_CONFIRMATION_TEXT,
-						NULL,
-						ResetLandscapeConfirmationCallback);
-					break;
+			case ETTW_LOWER_LAND: // Lower land button
+				HandlePlacePushButton(this, ETTW_LOWER_LAND, ANIMCURSOR_LOWERLAND, HT_POINT);
+				this->last_user_action = widget;
+				break;
+
+			case ETTW_RAISE_LAND: // Raise land button
+				HandlePlacePushButton(this, ETTW_RAISE_LAND, ANIMCURSOR_RAISELAND, HT_POINT);
+				this->last_user_action = widget;
+				break;
+
+			case ETTW_LEVEL_LAND: // Level land button
+				HandlePlacePushButton(this, ETTW_LEVEL_LAND, SPR_CURSOR_LEVEL_LAND, HT_POINT | HT_DIAGONAL);
+				this->last_user_action = widget;
+				break;
+
+			case ETTW_PLACE_ROCKS: // Place rocks button
+				HandlePlacePushButton(this, ETTW_PLACE_ROCKS, SPR_CURSOR_ROCKY_AREA, HT_RECT);
+				this->last_user_action = widget;
+				break;
+
+			case ETTW_PLACE_DESERT: // Place desert button (in tropical climate)
+				HandlePlacePushButton(this, ETTW_PLACE_DESERT, SPR_CURSOR_DESERT, HT_RECT);
+				this->last_user_action = widget;
+				break;
+
+			case ETTW_PLACE_OBJECT: // Place transmitter button
+				if (HandlePlacePushButton(this, ETTW_PLACE_OBJECT, SPR_CURSOR_TRANSMITTER, HT_RECT)) {
+					ShowBuildObjectPicker(this);
+					this->last_user_action = widget;
+				}
+				break;
+
+			case ETTW_INCREASE_SIZE:
+			case ETTW_DECREASE_SIZE: { // Increase/Decrease terraform size
+				int size = (widget == ETTW_INCREASE_SIZE) ? 1 : -1;
+				this->HandleButtonClick(widget);
+				size += _terraform_size;
+
+				if (!IsInsideMM(size, 1, 8 + 1)) return;
+				_terraform_size = size;
+
+				SndPlayFx(SND_15_BEEP);
+				this->SetDirty();
+				break;
 			}
+
+			case ETTW_NEW_SCENARIO: // gen random land
+				this->HandleButtonClick(widget);
+				ShowCreateScenario();
+				break;
+
+			case ETTW_RESET_LANDSCAPE: // Reset landscape
+				ShowQuery(STR_QUERY_RESET_LANDSCAPE_CAPTION, STR_RESET_LANDSCAPE_CONFIRMATION_TEXT, NULL, ResetLandscapeConfirmationCallback);
+				break;
+
+			default: NOT_REACHED();
 		}
 	}
 
@@ -719,7 +696,37 @@ struct ScenarioEditorLandscapeGenerationWindow : Window {
 
 	virtual void OnPlaceObject(Point pt, TileIndex tile)
 	{
-		_place_proc(tile);
+		switch (this->last_user_action) {
+			case ETTW_DEMOLISH: // Demolish aka dynamite button
+				PlaceProc_DemolishArea(tile);
+				break;
+
+			case ETTW_LOWER_LAND: // Lower land button
+				CommonRaiseLowerBigLand(tile, 0);
+				break;
+
+			case ETTW_RAISE_LAND: // Raise land button
+				CommonRaiseLowerBigLand(tile, 1);
+				break;
+
+			case ETTW_LEVEL_LAND: // Level land button
+				VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_LEVEL_AREA);
+				break;
+
+			case ETTW_PLACE_ROCKS: // Place rocks button
+				VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_CREATE_ROCKS);
+				break;
+
+			case ETTW_PLACE_DESERT: // Place desert button (in tropical climate)
+				VpStartPlaceSizing(tile, VPM_X_AND_Y, DDSP_CREATE_DESERT);
+				break;
+
+			case ETTW_PLACE_OBJECT: // Place transmitter button
+				PlaceProc_Object(tile);
+				break;
+
+			default: NOT_REACHED();
+		}
 	}
 
 	virtual void OnPlaceDrag(ViewportPlaceMethod select_method, ViewportDragDropSelectionProcess select_proc, Point pt)
@@ -748,9 +755,25 @@ struct ScenarioEditorLandscapeGenerationWindow : Window {
 	{
 		this->RaiseButtons();
 		this->SetDirty();
+		DeleteWindowById(WC_BUILD_OBJECT, 0);
 	}
+
+	static Hotkey<ScenarioEditorLandscapeGenerationWindow> terraform_editor_hotkeys[];
 };
 
+Hotkey<ScenarioEditorLandscapeGenerationWindow> ScenarioEditorLandscapeGenerationWindow::terraform_editor_hotkeys[] = {
+	Hotkey<ScenarioEditorLandscapeGenerationWindow>('D' | WKC_GLOBAL_HOTKEY, "dynamite", ETTW_DEMOLISH),
+	Hotkey<ScenarioEditorLandscapeGenerationWindow>('Q' | WKC_GLOBAL_HOTKEY, "lower", ETTW_LOWER_LAND),
+	Hotkey<ScenarioEditorLandscapeGenerationWindow>('W' | WKC_GLOBAL_HOTKEY, "raise", ETTW_RAISE_LAND),
+	Hotkey<ScenarioEditorLandscapeGenerationWindow>('E' | WKC_GLOBAL_HOTKEY, "level", ETTW_LEVEL_LAND),
+	Hotkey<ScenarioEditorLandscapeGenerationWindow>('R', "rocky", ETTW_PLACE_ROCKS),
+	Hotkey<ScenarioEditorLandscapeGenerationWindow>('T', "desert", ETTW_PLACE_DESERT),
+	Hotkey<ScenarioEditorLandscapeGenerationWindow>('O', "object", ETTW_PLACE_OBJECT),
+	HOTKEY_LIST_END(ScenarioEditorLandscapeGenerationWindow)
+};
+
+Hotkey<ScenarioEditorLandscapeGenerationWindow> *_terraform_editor_hotkeys = ScenarioEditorLandscapeGenerationWindow::terraform_editor_hotkeys;
+
 static const WindowDesc _scen_edit_land_gen_desc(
 	WDP_AUTO, 0, 0,
 	WC_SCEN_LAND_GEN, WC_NONE,
@@ -763,12 +786,11 @@ Window *ShowEditorTerraformToolbar()
 	return AllocateWindowDescFront<ScenarioEditorLandscapeGenerationWindow>(&_scen_edit_land_gen_desc, 0);
 }
 
-void ShowEditorTerraformToolbarWithTool(uint16 key, uint16 keycode)
+EventState TerraformToolbarEditorGlobalHotkeys(uint16 key, uint16 keycode)
 {
-	Window *w = FindWindowById(WC_SCEN_LAND_GEN, 0);
-
-	if (w == NULL) w = ShowEditorTerraformToolbar();
-	if (w == NULL) return;
-
-	w->OnKeyPress(key, keycode);
+	int num = CheckHotkeyMatch<ScenarioEditorLandscapeGenerationWindow>(_terraform_editor_hotkeys, keycode, NULL, true);
+	if (num == -1) return ES_NOT_HANDLED;
+	Window *w = ShowEditorTerraformToolbar();
+	if (w == NULL) return ES_NOT_HANDLED;
+	return w->OnKeyPress(key, keycode);
 }
